"""plugins template tag root hooks.
"""
from django.conf import settings
from django.db.models.loading import get_app
from django import template
from inspect import getargspec
from django.template.context import Context
from django.utils.functional import curry
from django.utils.encoding import smart_str
from django.template import loader, VariableDoesNotExist, Variable, Node
from django.template import TemplateDoesNotExist, TemplateSyntaxError
from django.core.cache.backends.locmem import CacheClass as LocalMemCache

TAG_KEYWORD_ARGUMENT_SEPARATOR = '='

## RED_FLAG: this feels wrong...
#from app_plugins.models import PluginPoint, Plugin
#from app_plugins.models import is_valid_label, construct_template_path
models = get_app('app_plugins')

APP_PLUGINS_CACHE_PARAMS = getattr(settings, 'APP_PLUGINS_CACHE_PARAMS',
                                     {'cull_frequency': 4,
                                      'max_entries': 3000,
                                     'timeout': 60*60*24*3, # 3 days
                                     })

app_plugin_apps_with_templates = LocalMemCache('localhost',
                                               APP_PLUGINS_CACHE_PARAMS)

# at import cache the app names for indexing
app_names = []
for app in settings.INSTALLED_APPS:
    name = app.split('.')[-1]
    if name not in app_names:
        app_names.append(name)
app_names = tuple(app_names)

def callback(func, variables, context, takes_context):
    """
    resolve an iterable of Variable objects into a list of args and a dict
    of keyword arguments. support full python style keyword argument
    processing::
        >>> def foo(a, b, c=1, d=2):
        ...     pass
        >>> foo(1, 2)
        >>> foo(1, b=2)
        >>> foo(b=2, a=1, d=3)
    """
    name = getattr(func, "_decorated_function", func).__name__
    params, varargs, varkw, defaults = getargspec(func)
    if takes_context:
        if params[0] == 'context':
            params.pop(0)
        else:
            raise TemplateSyntaxError(
                "Any tag function decorated with takes_context=True "
                "must have a first argument of 'context'")
    num_defaults = len(defaults)
    num_params = len(params)
    num_req = num_params - num_defaults

    args = []
    kwdargs = {}
    found_kwd = False
    for variable in variables:
        if not found_kwd:
            try:
                args.append(variable.resolve(context))
            except VariableDoesNotExist:
                if variable.var.count(TAG_KEYWORD_ARGUMENT_SEPARATOR) != 1:
                    raise
                found_kwd = True
        if found_kwd:
            try:
                var, path = variable.var.split(TAG_KEYWORD_ARGUMENT_SEPARATOR)
            except ValueError:
                raise TemplateSyntaxError(
                    "Expected keyword assignemnt, found '%s' instead" %
                    variable.var)
            if params and not varkw and name not in params:
                raise TemplateSyntaxError(
                    "%s got an unexpected keyword argument '%s'" % (name, var))
            if var in kwdargs:
                raise TemplateSyntaxError(
                    "%s got multiple values for keyword argument '%s'" %
                    (name, var))
            kwdargs[smart_str(var)] = Variable(path).resolve(context)

    num_args = len(args)
    num_kwds = len(kwdargs)
    num_all = num_args + num_kwds

    if ((num_args > num_params and not varargs) or
        (num_all> num_params and not varkw)):
        raise TemplateSyntaxError(
                "%s takes at most %s arguments. (%s given)" % (
                    name, num_params, num_all) )
    if num_args != num_req:
        if num_args > num_req:
            # some args are kwd args (maybe multiple keyword error)
            if not varargs:
                allowed = set(params[num_args:])
                not_allowed = set(kwdargs) - allowed
                if not_allowed:
                    raise TemplateSyntaxError(
                        "%s got multiple values for keyword arguments: %s" % (
                            name, ", ".join(not_allowed) ))
        elif not varkw:
            # not enough required parameters error
            required = set(params[num_args:-num_default])
            missing = required - set(kwdargs)
            if missing:
                raise TemplateSyntaxError(
                    "%s takes at least %s non-keyword arguments (%s given)" % (
                        name, num_req, num_args))
    if takes_context:
        args.insert(0, context)
    return func(*args, **kwdargs)

def compiler(node_class, parser, token):
    bits = token.split_contents()[1:]
    return node_class(bits)

def inclusion_kwdtag(register, file_name, context_class=Context,
                      takes_context=False):
        def dec(func):
            class InclusionKwdNode(Node):
                def __init__(self, vars_to_resolve):
                    self.vars_to_resolve = map(Variable, vars_to_resolve)

                def render(self, context):
                    new_context = callback(func, self.vars_to_resolve, context,
                                           takes_context)
                    if not getattr(self, 'nodelist', False):
                        if (not isinstance(file_name, basestring) and
                            is_iterable(file_name)):
                            t = loader.select_template(file_name)
                        else:
                            t = loader.get_template(file_name)
                        self.nodelist = t.nodelist
                    res = self.nodelist.render(context_class(new_context,
                            autoescape=context.autoescape))
                    context.pop() # local context
                    context.pop() # args
                    context.pop() # callback context (or empty)
                    return res

            compile_func = curry(compiler, InclusionKwdNode)
            compile_func.__doc__ = func.__doc__
            register.tag(getattr(func, "_decorated_function", func).__name__,
                         compile_func)
            return func
        return dec

register = template.Library()

@register.filter
def template_exists(templ):
    if templ is None: return False
    try:
        #loader.get_template(templ)
        loader.find_template_source(templ)
    except TemplateDoesNotExist:
        return False
    return True

def validate_name(name):
    ## red_flag: turn into a string
    if not models.is_valid_label(name):
        raise TemplateSyntaxError, "invalid plugin point name '%s'." % name

@inclusion_kwdtag(register, "app_plugins/app_plugin.html", takes_context=True)
def app_plugin(context, app, name, plugin=None, user=None, args=None,
               ext='.html', **extra_args):
    validate_name(app)
    validate_name(name)
    if plugin is None:
        try:
            plugin = models.Plugin.objects.get(label=u'.'.join([app, name]))
        except models.Plugin.DoesNotExist:
            pass
    nc = context

    if args is None: args = extra_args
    else: args.update(extra_args)

    template = ''
    if plugin is None:
        template = models.construct_template_path(app, name, ext)
        nc.push()
    else:
        nc.update(plugin.call(nc, user, **args))
        template = plugin.template
    nc.update(args)
    nc.push()
    nc['app_plugin'] = plugin
    nc['app_plugin_app'] = app
    nc['app_plugin_point'] = name
    nc['app_plugin_args'] = args
    nc['app_plugin_user'] = user
    nc['app_plugin_template'] = template
    return nc

@inclusion_kwdtag(register, "app_plugins/plugin_point.html", takes_context=True)
def plugin_point(context, name, point=None, user=None, ext='.html', **args):
    validate_name(name)
    if point is None:
        try:
            point = models.PluginPoint.objects.select_related().get(label=name)
        except models.PluginPoint.DoesNotExist:
            pass
    nc = context
    plugins = None
    if point is None:
        apps = app_plugin_apps_with_templates.get(name+ext, None)
        if apps is None:
            tpls = ((app, models.construct_template_path(app, name, ext))
                    for app in app_names)
            apps = [ app for app, tpl in tpls if template_exists(tpl) ]
            app_plugin_apps_with_templates.set(name+ext, apps)
        nc.push()
    else:
        nc.update(point.call(nc, user, **args))
        plugins = point.get_plugins(user)
        apps = [ p.app for p in plugins ]
    nc.update(args)
    nc.push()
    nc['app_plugin_ext'] = ext
    nc['app_plugin_point'] = name
    nc['app_plugin_apps'] = apps
    nc['app_plugin_args'] = args
    nc['app_plugin_user'] = user
    nc['app_plugin_plugins'] = plugins
    return nc



from payment.modules.purchaseorder.models import PurchaseOrder
from django.contrib import admin

class PurchaseOrderOptions(admin.ModelAdmin):
    list_display = ('po_number', 'balance_due', 'order_link')

admin.site.register(PurchaseOrder, PurchaseOrderOptions)





from decimal import Decimal
from django.db import models
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext, ugettext_lazy as _
from l10n.utils import moneyfmt
from satchmo_store.shop.models import Order

class PurchaseOrder(models.Model):
    po_number = models.CharField(_('Customer PO Number'), max_length=20)
    order = models.ForeignKey(Order)
    balance = models.DecimalField(_("Outstanding Balance"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    paydate = models.DateField(_('Paid on'), blank=True, null=True)
    notes = models.TextField(_('Notes'), blank=True, null=True)
    
    def __unicode__(self):
        return "PO: #%s" % self.po_number

    def balance_due(self):
        if self.balance:
            b = self.balance
        else:
            b = Decimal('0.00')
            
        return moneyfmt(b)

    def order_link(self):
        return mark_safe(u'<a href="/admin/shop/order/%i/">%s #%i (%s)</a>' % (
            self.order.id,
            ugettext('Order'), 
            self.order.id, 
            moneyfmt(self.order.total)))
            
    order_link.allow_tags = True
    
    def save(self):
        if self.balance is None:
            self.balance = self.order.balance
        super(PurchaseOrder, self).save()

import config
PAYMENT_PROCESSOR=True


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#import os
#import traceback
import sys

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from notmm.controllers.wsgi             import BaseController, WSGIController
from notmm.utils.django_settings        import SettingsProxy
from notmm.utils.wsgilib                import HTTPRequest, HTTPResponse
from notmm.utils.django_compat          import RegexURLResolver, get_resolver

settings = SettingsProxy(autoload=True).settings
class RegexUrlResolverTestCase(unittest.TestCase):
    
    def setUp(self):
        self.resolver_func = get_resolver
        self.urlconf = settings.ROOT_URLCONF
    def test_get_resolver(self):
        result = self.resolver_func(self.urlconf)
        self.failIfEqual(result, None)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2007-2012 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
# <LICENSE=ISC>
"""Basic logging utilities"""
import logging

__all__ = ['configure_logging', 'LOGGING_ERROR_LOG']

LOGGING_ERROR_LOG = '/var/log/django.log'

def configure_logging(logger, level=10, error_log=LOGGING_ERROR_LOG, handler=logging.FileHandler):
    logging.basicConfig(format='%(levelname)s:%(message)s', 
        level=logging.DEBUG)
    logger.addHandler(handler(error_log))
    
    return logger



from django.utils.translation import ugettext_lazy, ugettext
from livesettings import *
from payment import signals, active_gateways
from satchmo_store.shop import get_satchmo_setting
from satchmo_utils import is_string_like, load_module
import logging
import signals

_ = ugettext_lazy

log = logging.getLogger('payment.config')

PAYMENT_GROUP = ConfigurationGroup('PAYMENT', _('Payment Settings'))

CRON_KEY = config_register(
    StringValue(PAYMENT_GROUP,
        'CRON_KEY',
        description=_("Cron Passkey"),
        help_text=_("Enter an authentication passkey to secure your recurring billing cron url."),
        default = "x1234replace_me")
)

ALLOW_URL_CRON = config_register(
    BooleanValue(PAYMENT_GROUP,
        'ALLOW_URL_REBILL',
        description=_("Allow URL Access to Cron for subscription rebills"),
        help_text=_("Do you want to allow remote url calls for subscription billing?"),
        default = False)
)

PAYMENT_LIVE = config_register(
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode.  This is the master switch, turn it off to force all payments into test mode."),
        default=False)
)

ORDER_EMAIL = config_register(
    BooleanValue(PAYMENT_GROUP,
        'ORDER_EMAIL_OWNER',
        description=_("Email owner?"),
        help_text=_("True if you want to email the owner on order"),
        default=False)
)
    
ORDER_EMAIL_EXTRA = config_register(
    StringValue(PAYMENT_GROUP,
        'ORDER_EMAIL_EXTRA',
        description=_("Extra order emails?"),
        requires=ORDER_EMAIL,
        help_text=_("Put all email addresses you want to email in addition to the owner when an order is placed."),
        default = "")
)

config_register_list(

    BooleanValue(PAYMENT_GROUP,
        'AUTH_EARLY',
        description=_("Early AUTH"),
        help_text=_("Where possible, Authenticate on the card entry page.  This causes an immediate $.01 AUTH and release, allowing errors with the card to show on the card entry page instead of on the confirmation page.  Note that this is only supported for payment modules that can do Authorizations.  It will be silently ignored for any other processors."),
        default=False),

    BooleanValue(PAYMENT_GROUP,
        'COUNTRY_MATCH',
        description=_("Country match required?"),
        help_text=_("If True, then customers may not have different countries for shipping and billing."),
        default=True),

    BooleanValue(PAYMENT_GROUP,
        'SSL',
        description=_("Enable SSL"),
        help_text=_("""This enables for generic pages like contact information capturing.  
    It does not set SSL for individual modules. 
    You must enable SSL for each payment module individually."""),
        default=False),

    DecimalValue(PAYMENT_GROUP,
        'MINIMUM_ORDER',
        description=_("Minimum Order"),
        help_text=_("""The minimum cart total before checkout is allowed."""),
        default="0.00"),

    BooleanValue(PAYMENT_GROUP,
        'STORE_CREDIT_NUMBERS',
        description=_("Save Credit Card Numbers"),
        help_text=_("If False, then the credit card will never be written to disk.  For PCI compliance, this is required unless you have your database server on a separate server not connected to the internet."),
        default=False),

    PositiveIntegerValue(PAYMENT_GROUP,
        'CC_NUM_YEARS',
        description=_("Number of years to display for CC expiration"),
        help_text=_("Number of years that will be added to today's year for the CC expiration drop down"),
        default=10),
    
    BooleanValue(PAYMENT_GROUP,
        'USE_DISCOUNTS',
        description=_("Use discounts"),
        help_text=_("""If disabled, customers will not be asked for any discount codes."""),
        default=True)
)

# --- helper functions ---

def credit_choices(settings=None, include_module_if_no_choices=False):
    choices = []
    keys = []
    for module, group in active_gateways():
        vals = config_choice_values(group, 'CREDITCHOICES')
        for key, label in vals:
            if not key in keys:
                keys.append(key)
                pair = (key, ugettext(label))
                choices.append(pair)
        if include_module_if_no_choices and not vals:
            key = config_value(group, 'KEY')
            label = config_value(group, 'LABEL')
            pair = (key, ugettext(label))
            choices.append(pair)
    return choices

def labelled_gateway_choices():
    choices = []
    for module, group in active_gateways():
        defaultlabel = module.split('.')[-1]
        label = _(config_value(group, 'LABEL', default = defaultlabel))
        choices.append((group, label))
    
    signals.payment_choices.send(None, choices=choices)
    return choices

def gateway_live(settings):
    if is_string_like(settings):
        settings = config_get_group(settings)
    
    try:    
        if config_value('PAYMENT', 'LIVE'):
            return settings['LIVE'].value
            
    except SettingNotSet:
        pass
        
    return False



""" Just tracks subscriptions, nothing more. """

from satchmo_ext.newsletter.models import Subscription
from django.utils.translation import ugettext as _
import logging

log = logging.getLogger('simple newsletter')

def is_subscribed(contact):
    return Subscription.email_is_subscribed(contact.email)

def update_contact(contact, subscribe, attributes={}):
    email = contact.email
    current = Subscription.email_is_subscribed(email)
    attributesChanged = False
    sub = None
    
    if attributes:
        sub, created = Subscription.objects.get_or_create(email=email)
        if created:
            attributesChanged = True
        else:
            oldAttr = [(a.name,a.value) for a in sub.attributes.all()]
            oldAttr.sort()
        sub.update_attributes(attributes)
        newAttr = [(a.name,a.value) for a in sub.attributes.all()]
        newAttr.sort()
        
        if not created:
            attributesChanged = oldAttr != newAttr
    
    if current == subscribe:
        if subscribe:
            if attributesChanged:
                result = _("Updated subscription for %(email)s.")
            else:
                result = _("Already subscribed %(email)s.")
                
        else:
            result = _("Already removed %(email)s.")
        
    else:
        if not sub:
            sub, created = Subscription.objects.get_or_create(email=email)
        sub.subscribed = subscribe
        sub.save()
        log.debug("Subscription now: %s" % sub)

        if subscribe:
            result = _("Subscribed: %(email)s")
        else:
            result = _("Unsubscribed: %(email)s")

    return result % { 'email' : email }



# Django settings for satchmo project.
# This is a recommended base setting for further customization
from django.conf.global_settings import *
import os

# Make this unique, and don't share it with anybody.
SECRET_KEY = '3+rl2paxh^e@^m9jso6x9w@qrmvb%y$qgbwj#gt8ijrs-)r)zu'

#import django
#django.setup()

DEBUG = True
DIRNAME = os.path.dirname(__file__)

ADMINS = (
     ('Etienne Robillard', 'erob@gthcfoundation.org'),
)

MANAGERS = ADMINS

DATABASE_ENGINE = 'sqlite3'    # 'postgresql', 'mysql', 'sqlite3' or 'ado_mssql'.
# The following variables should be configured in your local_settings.py file
#DATABASE_NAME = ''            # Or path to database file if using sqlite3.
#DATABASE_USER = ''            # Not used with sqlite3.
#DATABASE_PASSWORD = ''        # Not used with sqlite3.
DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.

# Local time zone for this installation. All choices can be found here
# http://www.postgresql.org/docs/current/static/datetime-keywords.html#DATETIME-TIMEZONE-SET-TABLE
TIME_ZONE = 'America/Montreal'

# Language code for this installation. All choices can be found here:
# http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes
# http://blogs.law.harvard.edu/tech/stories/storyReader$15
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
# Image files will be stored off of this path
#
# If you are using Windows, recommend using normalize_path() here
#
# from satchmo_utils.thumbnail import normalize_path
# MEDIA_ROOT = normalize_path(os.path.join(DIRNAME, 'static/'))
MEDIA_ROOT = os.path.join(DIRNAME, 'static/')

# URL that handles the media served from MEDIA_ROOT.
# Example: "http://media.lawrence.com"
MEDIA_URL = "http://localhost/satchmo_media/"

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = MEDIA_URL + 'admin/'



# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    #"django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    #"django.middleware.csrf.CsrfMiddleware",
    #"django.middleware.doc.XViewMiddleware",
    #"threaded_multihost.middleware.ThreadLocalMiddleware",
    #"satchmo_store.shop.SSLMiddleware.SSLRedirect",
    #"satchmo_ext.recentlist.middleware.RecentProductMiddleware",
]

#this is used to add additional config variables to each request
# NOTE: If you enable the recent_products context_processor, you MUST have the
# 'satchmo_ext.recentlist' app installed.
TEMPLATE_CONTEXT_PROCESSORS = (
'satchmo_store.shop.context_processors.settings',
'django.core.context_processors.auth',
'django.core.context_processors.media',
#'django.core.context_processors.csrf'
#'satchmo_ext.recentlist.context_processors.recent_products',
)

ROOT_URLCONF = 'store.urls'

INSTALLED_APPS = (
    
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sites',
    'django.contrib.admin',

    #'satchmo_admin',
    #'django.contrib.comments',
    'django.contrib.sessions',
    'django.contrib.sitemaps',
    'satchmo_store.shop',
    'satchmo_store',
    #'registration',
    'sorl.thumbnail',
    'keyedcache',
    'livesettings',
    'l10n',
    'satchmo_utils.thumbnail',
    'satchmo_store.contact',
    'tax',
    'tax.modules.no',
    'tax.modules.area',
    'tax.modules.percent',
    'shipping',
    #'contact.supplier',
    #'shipping.modules.tiered',
    'satchmo_ext.newsletter',
    #'satchmo_ext.recentlist',
    #'testimonials',
    'product',
    #'satchmo_ext.product_feeds',
    #'satchmo_ext.brand',
    'payment',
    'payment.modules.dummy',
    #'payment.modules.purchaseorder',
    #'payment.modules.giftcertificate',
    #'satchmo_ext.wishlist',
    #'satchmo_ext.upsell',
    #'satchmo_ext.productratings',
    'satchmo_ext.satchmo_toolbar',
    'satchmo_utils',
    #'shipping.modules.tieredquantity',
    #'satchmo_ext.tieredpricing',
    #'typogrify',
    #'debug_toolbar',
    'app_plugins',
    #'localsite',
)

AUTHENTICATION_BACKENDS = (
    'satchmo_store.accounts.email-auth.EmailBackend',
    'django.contrib.auth.backends.ModelBackend',
)

#### Satchmo unique variables ####
#from django.conf.urls.defaults import patterns, include
SATCHMO_SETTINGS = {
    'SHOP_BASE' : '/shop/',
    'MULTISHOP' : False,
    #'SHOP_URLS' : patterns('satchmo_store.shop.views',)
}

ENABLE_BEAKER = False



import gtk

import sys

from gazpacho.loader.loader import ObjectBuilder

def on_new__activated(action):
    print 'on new'

def on_quit__activated(action):
    gtk.main_quit()

def on_window1__destroy(window):
    gtk.main_quit()

def main(args):
    wt = ObjectBuilder ('uimanager.glade')
    wt.signal_autoconnect(globals())
    if '--benchmark' in args:
        while gtk.events_pending():
            gtk.main_iteration_do()
        return

    window = wt.get_widget('window1')
    window.set_size_request(400, 300)
    window.show()
    gtk.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv))



from django.db.models import signals
from commands.sync_plugins import sync_app_plugins
from app_plugins import models as sender_app

def do_sync(*args, **kwdargs):
    sync_app_plugins(verbosity=kwdargs.get("verbosity", 1))

signals.post_syncdb.connect(do_sync, sender=sender_app)

if __name__ == "__main__":
    sync_app_plugins()



from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext
from django.views.decorators.cache import never_cache
from livesettings import config_get_group
from payment.utils import pay_ship_save, get_processor_by_key
from satchmo_store.shop.models import Cart
from satchmo_store.shop.models import Order, Contact, OrderPayment
from satchmo_utils.dynamic import lookup_url, lookup_template

import logging

log = logging.getLogger('autosuccess.views')

def one_step(request):
    payment_module = config_get_group('PAYMENT_AUTOSUCCESS')

    #First verify that the customer exists
    try:
        contact = Contact.objects.from_request(request, create=False)
    except Contact.DoesNotExist:
        url = lookup_url(payment_module, 'satchmo_checkout-step1')
        return HttpResponseRedirect(url)
    #Verify we still have items in the cart
    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0:
        template = lookup_template(payment_module, 'shop/checkout/empty_cart.html')
        return render_to_response(template,
                                  context_instance=RequestContext(request))

    # Create a new order
    newOrder = Order(contact=contact)
    pay_ship_save(newOrder, tempCart, contact,
        shipping="", discount="")
        
    request.session['orderID'] = newOrder.id
    
    processor = get_processor_by_key('PAYMENT_AUTOSUCCESS')
    processor.prepare_data(newOrder)
    payment = processor.process(newOrder)
        
    tempCart.empty()
    success = lookup_url(payment_module, 'satchmo_checkout-success')
    return HttpResponseRedirect(success)
one_step = never_cache(one_step)



"""Simple wrapper for standard checkout as implemented in payment.views"""

from django import http
from forms import PurchaseorderPayShipForm
from livesettings import config_get_group
from payment.views import confirm, payship
from satchmo_utils.dynamic import lookup_url
import logging

log = logging.getLogger('purchaseorder.views')

settings = config_get_group('PAYMENT_PURCHASEORDER')
    
def pay_ship_info(request):
    return payship.base_pay_ship_info(
        request, 
        settings, 
        purchaseorder_process_form, 
        'shop/checkout/purchaseorder/pay_ship.html')
        
def confirm_info(request):
    return confirm.credit_confirm_info(
        request, 
        settings, 
        template='shop/checkout/purchaseorder/confirm.html')

def purchaseorder_process_form(request, contact, working_cart, payment_module):
    log.debug('purchaseorder_process_form')
    if request.method == "POST":
        log.debug('handling POST')
        new_data = request.POST.copy()
        form = PurchaseorderPayShipForm(request, payment_module, new_data)
        if form.is_valid():
            form.save(request, working_cart, contact, payment_module)
            url = lookup_url(payment_module, 'satchmo_checkout-step3')
            return (True, http.HttpResponseRedirect(url))
    else:
        log.debug('new form')
        form = PurchaseorderPayShipForm(request, payment_module)

    return (False, form)



#!/usr/bin/env python
# Copyright (c) Etienne Robillard 2007-2009 <robillard.etienne@gmail.com>

from notmm.utils.wsgilib import HTTPRedirectResponse
from notmm.utils.template import direct_to_template

__all__ = ('handle302', 'handle404', 'handle500')

def handle302(request, **kwargs):
    location = kwargs.get('location', '/')
    return HTTPRedirectResponse(location=location)

def handle500(request, template_name='500.mako'):
    return direct_to_template(request, template_name, status=500)
 
def handle404(request, template_name='404.mako'):
    return direct_to_template(request, template_name, status=404)
    



"""Provides the ability to overload forms to provide multiple possible 
responses to a form.

For example, the product add form.  You can add to the cart, or to the 
wishlist.  The view here just looks in the formdata to determine whether
 to send the request to the cart_add or the wishlist_add view.
"""
from satchmo_store.shop.models import Cart
from satchmo_store.shop.signals import cart_add_view
from satchmo_store.shop.views import cart

def smart_add(request):
    """Redirect the request to cart_add (default) or whatever gets returned by
    the cart_add_view signal."""
    
    method={'view': cart.add}
    cart_add_view.send(Cart, request=request, method=method)
    
    return method['view'](request)
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2018 Etienne Robillard <tkadm30@yandex.com>
# All rights reserved.
#
# This file is part of the notmm distribution.
# Please review the LICENSE file for details.

"""DataStore API

The DataStore API provides a simplified environment for storing
user/system data. It should be noted that the current state of 
the API is still in an early alpha stage and is subject to change.
"""

__all__ = ['BaseStore', 'DataStore']

import posixpath
import copy
from notmm.utils.configparse import is_uppercase


class BaseStore(object):
    """Base class for storing and loading objects."""

    def __init__(self):
        pass

    def count(self):
        """
        count the number of objects in UPPER_CASE found
        in __dict__ and return that number. Note that
        only UPPER_CASE keys are being counted.
        """
        return len([ key for key in self.__dict__ if is_uppercase(key) ])  

    def clear(self):
        """ remove everything that looks like a setting option """
        for key, val in self.__dict__.items():
            if is_uppercase(key):
                del self.__dict__[key]
        # reset the initialized object..
        #self.initialized = False
        # and always return something..
        return self


class DataStore(BaseStore):
    """Wrapper to store local data in a single thread"""
   
    _initialized = False
    _default_modname = 'local_settings'
    
    def __init__(self, modname, autoload=True):
        """Implements the __init__ method for ThreadLocalStore instances.
        
        Automatically load the objects contained in ``modname`` if autoload is True.
        """

        # This init code was borrowed from the 
        # _threading_local module.
        self._modname = modname
        self._local_data = {}
        
        if autoload and not self._initialized:
            self.loads(modname)

    def loads(self, modname=None):
        """ 
        Load objects from `modname` using the `__import__`
        builtin function.
        """
        if not modname:
            #raise ValueError("Missing modname parameter.")
            modname = self.default_modname

        head, tail = posixpath.splitext(modname)
        module_name = tail.lstrip('.')
        # this must handle both absolute and relative imports
        m_obj = __import__(modname, {}, {}, fromlist=[module_name])
        # Copy the precious settings into our thread-safe 
        # storing class.
        if m_obj:
            for key in m_obj.__dict__:
                self._local_data[key] = m_obj.__dict__[key]
            self.__dict__ = self._local_data.copy()
            self.__dict__['_modname'] = modname
            self.__dict__['_initialized'] = True
        return self

    def __iter__(self):
        return iter(self.__dict__)

    def __next__(self):
        for item in self.__dict__.iteritems():
            yield item
    
    def __getitem__(self, name):
        try:
            return self.__dict__[name]
        except KeyError:
            raise

    def __setitem__(self, name, value):
        try:
            self.__dict__[name] = value
        except Exception:
            raise

    def __str__(self):
        return "%s" % self._modname

ThreadLocalStore = DataStore # for backward-compatibility




from django import http
from django.contrib.sites.models import Site
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from forms import GiftCertCodeForm, GiftCertPayShipForm
from models import GiftCertificate, GIFTCODE_KEY
from livesettings import config_get_group
from satchmo_store.shop.models import Order
from payment.utils import pay_ship_save, get_or_create_order
from payment.views import confirm, payship
from satchmo_utils.dynamic import lookup_url
from django.contrib.sites.models import Site
import logging

log = logging.getLogger("giftcertificate.views")

gc = config_get_group('PAYMENT_GIFTCERTIFICATE')
    
def giftcert_pay_ship_process_form(request, contact, working_cart, payment_module):
    if request.method == "POST":
        new_data = request.POST.copy()
        form = GiftCertPayShipForm(request, payment_module, new_data)
        if form.is_valid():
            data = form.cleaned_data

            # Create a new order.
            newOrder = get_or_create_order(request, working_cart, contact, data)            
            newOrder.add_variable(GIFTCODE_KEY, data['giftcode'])
            
            request.session['orderID'] = newOrder.id

            url = None
            if not url:
                url = lookup_url(payment_module, 'satchmo_checkout-step3')
                
            return (True, http.HttpResponseRedirect(url))
    else:
        form = GiftCertPayShipForm(request, payment_module)

    return (False, form)

    
def pay_ship_info(request):
    return payship.base_pay_ship_info(request, 
        gc, 
        giftcert_pay_ship_process_form,
        template="shop/checkout/giftcertificate/pay_ship.html")
    
def confirm_info(request, template="shop/checkout/giftcertificate/confirm.html"):
    try:
        order = Order.objects.get(id=request.session['orderID'])
        giftcert = GiftCertificate.objects.from_order(order)
    except (Order.DoesNotExist, GiftCertificate.DoesNotExist, KeyError):
        giftcert = None
           
    controller = confirm.ConfirmController(request, gc)
    controller.templates['CONFIRM'] = template
    controller.extra_context={'giftcert' : giftcert}
    controller.confirm()
    return controller.response

def check_balance(request):
    if request.method == "GET":        
        code = request.GET.get('code', '')
        if code:
            try:
                gc = GiftCertificate.objects.get(code=code, 
                    value=True, 
                    site=Site.objects.get_current())
                success = True
                balance = gc.balance
            except GiftCertificate.DoesNotExist:
                success = False
        else:
            success = False
        
        ctx = RequestContext(request, {
            'code' : code,
            'success' : success,
            'balance' : balance,
            'giftcertificate' : gc
        })
    else:
        form = GiftCertCodeForm()
        ctx = RequestContext(request, {
            'code' : '',
            'form' : form
        })
    return render_to_response('giftcertificate/balance.html',
                              context_instance=ctx)



"""Protx Form"""
from django import forms
from django.utils.translation import ugettext as _
from payment.forms import CreditPayShipForm, MONTHS
from payment.modules.protx.config import REQUIRES_ISSUE_NUMBER
import logging

log = logging.getLogger('payment.protx.forms')

class ProtxPayShipForm(CreditPayShipForm):
    """Adds fields required by Prot/X to the Credit form."""
    
    card_holder = forms.CharField(max_length=75, required=False)
    month_start = forms.ChoiceField(choices=[(1, '--')]+MONTHS, required=False)
    year_start = forms.ChoiceField(required=False)
    issue_num = forms.CharField(max_length=2, required=False)
    
    def __init__(self, request, paymentmodule, *args, **kwargs):
        super(ProtxPayShipForm, self).__init__(request, paymentmodule, *args, **kwargs)
        cf = self.fields['card_holder']
        if (not cf.initial) or cf.initial == "":
            user = request.user
            if user and user.is_authenticated() and user.contact_set.count() > 0:
                cf.initial = self.tempContact.full_name
        self.requires_issue_number = REQUIRES_ISSUE_NUMBER
        self.fields['year_start'].choices = self.fields['year_expires'].choices

    def save(self, request, cart, contact, payment_module, data=None):
        """Save the order and the credit card details."""
        super(ProtxPayShipForm, self).save(request, cart, contact, payment_module)
        if data is None:
            data = self.cleaned_data 
        log.debug("data: %s", data)                       
        card_holder=data.get('card_holder', '')
        if not card_holder:
            card_holder = contact.full_name
        self.cc.card_holder = card_holder
        month_start = data.get('month_start', None)
        year_start = data.get('year_start', None)
        if month_start:
            try:
                month_start = int(month_start)
            except ValueError:
                log.warn("Could not parse month_start int from %s on order for order: %s", month_start, self.order)
                month_start = 1
                # TODO: raise some error to be caught by processor
        else:
            month_start=None
            
        if year_start:
            try:
                year_start = int(year_start)
            except ValueError:
                log.warn("Could not parse year_start int from %s on order for order: %s", year_start, self.order)
                year_start = 1
        else:
            year_start=None
        
        issue_num = data.get('issue_num', "")
        
        self.cc.start_month=month_start
        self.cc.start_year=year_start
        self.cc.issue_num=issue_num
        self.cc.save()

    def clean_card_holder(self):
        ch = self.cleaned_data['card_holder']
        if (not ch) or ch == "":
            fn = self.tempContact.full_name
            self.cleaned_data['card_holder'] = fn
            log.debug('Setting card_holder to contact full name: %s', fn)
        else:
            log.debug('Card holder OK')

    def clean_month_start(self):
        data = self.cleaned_data
        
        self._maybe_require(data, 'month_start', _('You must provide a starting month when using this type of card.'))

        if data['month_start']:
            try:
                v = int(self.cleaned_data['month_start'])
            except ValueError:
                raise forms.ValidationError(_("This must be a number"))
            
            if not v>0 and v<13:
                raise forms.ValidationError(_("Out of range, must be 1-12"))
            
    def clean_year_start(self):
        data = self.cleaned_data
        self._maybe_require(data, 'credit_type', _('You must provide a starting year when using this type of card.'))
        
        if data['year_start']:
            try:
                v = int(self.cleaned_data['year_start'])
            except ValueError:
                raise forms.ValidationError(_("This must be a number"))

            if not v>0 and v<13:
                raise forms.ValidationError(_("Out of range, must be 1-12"))

    def clean_issue_num(self):
        data = self.cleaned_data
        self._maybe_require(data, 'issue_num', _('You must provide an issue number when using this type of card.'))

    def _maybe_require(self, data, field, message):
        if data['credit_type'] in REQUIRES_ISSUE_NUMBER and not (data[field]):
            raise forms.ValidationError(message)



from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s
_strings = (gettext('CreditCard'), gettext('Credit Card'))

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_CYBERSOURCE', 
    _('Cybersource.net Payment Settings'), 
    ordering=102)

config_register_list(

    StringValue(PAYMENT_GROUP, 
        'CONNECTION',
        description=_("WSDL URL"),
        help_text=_("""This is the address to submit live transactions."""),
        default='https://ics2ws.ic3.com/commerce/1.x/transactionProcessor/CyberSourceTransaction_1.26.wsdl'),

    StringValue(PAYMENT_GROUP, 
        'CONNECTION_TEST',
        description=_("Submit to Test WSDL URL"),
        help_text=("""This is the address to submit test transactions"""),
        default='https://ics2wstest.ic3.com/commerce/1.x/transactionProcessor/CyberSourceTransaction_1.26.wsdl'),

    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the checkout pages?"), 
        default=False),
    
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
    
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.cybersource'),
    
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'CYBERSOURCE'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Credit Cards',
        help_text = _('This will be passed to the translation utility')),
        
    StringValue(PAYMENT_GROUP,
        'CURRENCY_CODE',
        description=_('Currency Code'),
        help_text=_('Currency code for Cybersource transactions.'),
        default = 'USD'),
        
    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = r'^credit/'),

    MultipleStringValue(PAYMENT_GROUP,
        'CREDITCHOICES',
        description=_('Available credit cards'),
        choices = (
            (('American Express', 'American Express')),
            (('Visa','Visa')),
            (('Mastercard','Mastercard')),
            (('Discover','Discover'))),
        default = ('Visa', 'Mastercard', 'Discover')),
    
    StringValue(PAYMENT_GROUP, 
        'MERCHANT_ID', 
        description=_('Your Cybersource merchant ID'),
        default=""),
    
    LongStringValue(PAYMENT_GROUP, 
        'TRANKEY', 
        description=_('Your Cybersource transaction key'),
        default=""),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



from datetime import datetime, timedelta
from decimal import Decimal
from django.http import HttpResponse
from django.utils.translation import ugettext, ugettext_lazy as _
from livesettings import config_get_group, config_value
from satchmo_store.shop.models import Order, OrderItem, OrderPayment
from satchmo_utils.views import bad_or_missing
import logging

log = logging.getLogger('payment.views.cron')

def cron_rebill(request=None):
    """Rebill customers with expiring recurring subscription products
    This can either be run via a url with GET key authentication or
    directly from a shell script.
    """
    #TODO: support re-try billing for failed transactions

    if request is not None:
        if not config_value('PAYMENT', 'ALLOW_URL_REBILL'):
            return bad_or_missing(request, _("Feature is not enabled."))
        if 'key' not in request.GET or request.GET['key'] != config_value('PAYMENT','CRON_KEY'):
            return HttpResponse("Authentication Key Required")

    expiring_subscriptions = OrderItem.objects.filter(expire_date__gte=datetime.now()).order_by('order', 'id', 'expire_date')
    for item in expiring_subscriptions:
        if item.product.is_subscription:#TODO - need to add support for products with trial but non-recurring
            if item.product.subscriptionproduct.recurring_times and item.product.subscriptionproduct.recurring_times + item.product.subscriptionproduct.get_trial_terms().count() == OrderItem.objects.filter(order=item.order, product=item.product).count():
                continue
            if item.expire_date == datetime.date(datetime.now()) and item.completed:
                if item.id == OrderItem.objects.filter(product=item.product, order=item.order).order_by('-id')[0].id:
                    #bill => add orderitem, recalculate total, porocess card
                    new_order_item = OrderItem(order=item.order, product=item.product, quantity=item.quantity, unit_price=item.unit_price, line_item_price=item.line_item_price)
                    #if product is recurring, set subscription end
                    if item.product.subscriptionproduct.recurring:
                        new_order_item.expire_date = item.product.subscriptionproduct.calc_expire_date()
                    #check if product has 2 or more trial periods and if the last one paid was a trial or a regular payment.
                    ordercount = item.order.orderitem_set.all().count()
                    if item.product.subscriptionproduct.get_trial_terms().count() > 1 and item.unit_price == item.product.subscriptionproduct.get_trial_terms(ordercount - 1).price:
                        new_order_item.unit_price = item.product.subscriptionproduct.get_trial.terms(ordercount).price
                        new_order_item.line_item_price = new_order_item.quantity * new_order_item.unit_price
                        new_order_item.expire_date = item.product.subscriptionproduct.get_trial_terms(ordercount).calc_expire_date()
                    new_order_item.save()
                    item.order.recalculate_total()
#                    if new_order_item.product.subscriptionproduct.is_shippable == 3:
#                        item.order.total = item.order.total - item.order.shipping_cost
#                        item.order.save()
                    payments = item.order.payments.all()[0]
                    #list of ipn based payment modules.  Include processors that use 3rd party recurring billing.
                    ipn_based = ['PAYPAL']
                    if not payments.payment in ipn_based and item.order.balance > 0:
                        #run card
                        #Do the credit card processing here & if successful, execute the success_handler
                        from livesettings import config_get_group
                        payment_module = config_get_group('PAYMENT_%s' % payments.payment)
                        credit_processor = payment_module.MODULE.load_module('processor')
                        processor = credit_processor.PaymentProcessor(payment_module)
                        processor.prepare_data(item.order)
                        result = processor.process()
        
                        if result.payment:
                            reason_code = result.payment.reason_code
                        else:
                            reason_code = "unknown"
                        log.info("""Processing %s recurring transaction with %s
                            Order #%i
                            Results=%s
                            Response=%s
                            Reason=%s""",
                            payment_module.LABEL.value,
                            payment_module.KEY.value,
                            item.order.id,
                            result.success,
                            reason_code,
                            result.message)

                        if result.success:
                            #success handler
                            item.order.add_status(status='New', notes = ugettext("Subscription Renewal Order successfully submitted"))
                            new_order_item.completed = True
                            new_order_item.save()
                            orderpayment = OrderPayment(order=item.order, amount=item.order.balance, payment=unicode(payment_module.KEY.value))
                            orderpayment.save()
    return HttpResponse()



"""A central mechanism for shop-wide settings which have defaults.

Repurposed from Sphene Community Tools: http://sct.sphene.net
"""

from django.conf import settings

satchmo_settings_defaults = {
    # Only settings for core `satchmo` applications are defined here,
    # (or global settings) -- all other defaults should be added using
    # the add_setting_defaults method !

    'SHOP_URLS' : [],
    'SHOP_BASE' : getattr(settings, 'SATCHMO_SHOP_BASE', '/shop/'),
    'MULTISHOP' : False    ,
    'CUSTOM_NEWSLETTER_MODULES' : [],
    'CUSTOM_SHIPPING_MODULES' : [],
    'CUSTOM_PRODUCT_MODULES' : [],
    'CUSTOM_TAX_MODULES' : [],
    'ALLOW_PRODUCT_TRANSLATIONS' : False,
    'COOKIE_MAX_SECONDS' : 60*60*24*30, #one month
    'CATEGORY_SLUG': 'category', # Used for the category url
    'PRODUCT_SLUG' : 'product', # Used for the product url
    }


def add_setting_defaults(newdefaults):
    """
    This method can be used by other applications to define their
    default values.
    
    newdefaults has to be a dictionary containing name -> value of
    the settings.
    """
    satchmo_settings_defaults.update(newdefaults)


def set_satchmo_setting(name, value):
    if not hasattr(settings, 'SATCHMO_SETTINGS'):
        settings.SATCHMO_SETTINGS = {}
    settings.SATCHMO_SETTINGS[name] = value
    

def get_satchmo_setting(name, default_value = None):
    if not hasattr(settings, 'SATCHMO_SETTINGS'):
        return satchmo_settings_defaults.get(name, default_value)

    return settings.SATCHMO_SETTINGS.get(name, satchmo_settings_defaults.get(name, default_value))




"""
Configuration items for the shop.
Also contains shopping cart and related classes.
"""

from decimal import Decimal, ROUND_CEILING
from django.contrib.sites.models import Site
from django.conf import settings
from django.core import urlresolvers
from django.db import models
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext, ugettext_lazy as _
from l10n.models import Country
from l10n.utils import moneyfmt
from livesettings import ConfigurationSettings, config_value
from payment.fields import PaymentChoiceCharField
from product.models import Discount, Product, Price, get_product_quantity_adjustments
from product.prices import PriceAdjustmentCalc, PriceAdjustment
from satchmo_store.contact.models import Contact
from satchmo_utils.fields import CurrencyField
from satchmo_utils.numbers import trunc_decimal
from shipping.fields import ShippingChoiceCharField
from tax.utils import get_tax_processor
import datetime
import keyedcache
import logging
import operator
import signals

log = logging.getLogger('satchmo_store.shop.models')

class NullConfig(object):
    """Standin for a real config when we don't have one yet."""

    def __init__(self):
        self.store_name = self.store_description = _("Test Store")
        self.store_email = self.street1 = self.street2 = self.city = self.state = self.postal_code = self.phone = ""
        self.site = self.country = None
        self.in_country_only = True
        self.sales_country = None

    def _options(self):
        return ConfigurationSettings()

    options = property(fget=_options)

    def __str__(self):
        return "Test Store - no configured store exists!"

class ConfigManager(models.Manager):
    def get_current(self, site=None):
        """Convenience method to get the current shop config"""
        if not site:
            site = Site.objects.get_current()

        site = site.id

        try:
            shop_config = keyedcache.cache_get("Config", site)
        except keyedcache.NotCachedError, nce:
            try:
                shop_config = self.get(site__id__exact=site)
                keyedcache.cache_set(nce.key, value=shop_config)
            except Config.DoesNotExist:
                log.warning("No Shop Config found, using test shop config for site=%s.", site)
                shop_config = NullConfig()

        return shop_config

class Config(models.Model):
    """
    Used to store specific information about a store.  Also used to
    configure various store behaviors
    """
    site = models.OneToOneField(Site, verbose_name=_("Site"), primary_key=True)
    store_name = models.CharField(_("Store Name"),max_length=100, unique=True)
    store_description = models.TextField(_("Description"), blank=True, null=True)
    store_email = models.EmailField(_("Email"), blank=True, null=True, max_length=75)
    street1=models.CharField(_("Street"),max_length=50, blank=True, null=True)
    street2=models.CharField(_("Street"), max_length=50, blank=True, null=True)
    city=models.CharField(_("City"), max_length=50, blank=True, null=True)
    state=models.CharField(_("State"), max_length=30, blank=True, null=True)
    postal_code=models.CharField(_("Zip Code"), blank=True, null=True, max_length=9)
    country=models.ForeignKey(Country, blank=False, null=False, verbose_name=_('Country'))
    phone = models.CharField(_("Phone Number"), blank=True, null=True, max_length=30)
    in_country_only = models.BooleanField(
        _("Only sell to in-country customers?"),
        default=True)
    sales_country = models.ForeignKey(
        Country, blank=True, null=True,
        related_name='sales_country',
        verbose_name=_("Default country for customers"))
    shipping_countries = models.ManyToManyField(
        Country,
        blank=True,
        verbose_name=_("Shipping Countries"),
        related_name="shop_configs")

    objects = ConfigManager()

    def _options(self):
        return ConfigurationSettings()

    options = property(fget=_options)

    def areas(self):
        """Get country areas (states/counties).  Used in forms."""
        if self.in_country_only:
            return self.sales_country.adminarea_set.filter(active=True)
        else:
            return None

    def countries(self):
        """Get country selections.  Used in forms."""
        if self.in_country_only:
            return Country.objects.filter(pk=self.sales_country.pk)
        else:
            return self.shipping_countries.filter(active=True)


    def _base_url(self, secure=False):
        prefix = "http"
        if secure:
            prefix += "s"
        return prefix + "://" + self.site.domain

    base_url = property(fget=_base_url)

    def save(self, **kwargs):
        keyedcache.cache_delete("Config", self.site.id)
        # ensure the default country is in shipping countries
        mycountry = self.country

        if mycountry:
            if not self.sales_country:
                log.debug("%s: No sales_country set, adding country of store, '%s'", self, mycountry)
                self.sales_country = mycountry

# This code doesn't work when creating a new site. At the time of creation, all of the necessary relationships
# aren't setup. I modified the load_store code so that it would create this relationship manually when running
# with sample data. This is a bit of a django limitation so I'm leaving this in here for now. - CBM
#            salescountry = self.sales_country
#            try:
#                need = self.shipping_countries.get(pk=salescountry.pk)
#            except Country.DoesNotExist:
#                log.debug("%s: Adding default country '%s' to shipping countries", self, salescountry.iso2_code)
#                self.shipping_countries.add(salescountry)
        else:
            log.warn("%s: has no country set", self)

        super(Config, self).save(**kwargs)
        keyedcache.cache_set("Config", self.site.id, value=self)

    def __unicode__(self):
        return self.store_name

    class Meta:
        verbose_name = _("Store Configuration")
        verbose_name_plural = _("Store Configurations")


class NullCart(object):
    """Standin for a real cart when we don't have one yet.  More convenient than testing for null all the time."""
    desc = None
    date_time_created = None
    customer = None
    total = Decimal("0")
    numItems = 0

    def add_item(self, *args, **kwargs):
        pass

    def remove_item(self, *args, **kwargs):
        pass

    def empty(self):
        pass

    def __str__(self):
        return "NullCart (empty)"

    def __iter__(self):
        return iter([])

    def __len__(self):
        return 0

class OrderCart(NullCart):
    """Allows us to fake a cart if we are reloading an order."""

    def __init__(self, order):
        self._order = order

    def _numItems(self):
        return self._order.orderitem_set.count()

    numItems = property(_numItems)

    def _cartitem_set(self):
        return self._order.orderitem_set

    cartitem_set = property(_cartitem_set)

    def _total(self):
        return self._order.balance

    total = property(_total)

    is_shippable = False

    def __str__(self):
        return "OrderCart (%i) = %i" % (self._order.id, len(self))

    def __len__(self):
        return self.numItems

class CartManager(models.Manager):

    def from_request(self, request, create=False, return_nullcart=True):
        """Get the current cart from the request"""
        cart = None
        try:
            contact = Contact.objects.from_request(request, create=False)
        except Contact.DoesNotExist:
            contact = None

        if 'cart' in request.session:
            cartid = request.session['cart']
            if cartid == "order":
                log.debug("Getting Order Cart from request")
                try:
                    order = Order.objects.from_request(request)
                    cart = OrderCart(order)
                except Order.DoesNotExist:
                    pass

            else:
                try:
                    cart = Cart.objects.get(id=cartid)
                except Cart.DoesNotExist:
                    log.debug('Removing invalid cart from session')
                    del request.session['cart']

        if isinstance(cart, NullCart) and not isinstance(cart, OrderCart) and contact is not None:
            carts = Cart.objects.filter(customer=contact)
            if carts.count() > 0:
                cart = carts[0]
                request.session['cart'] = cart.id

        if not cart:
            if create:
                site = Site.objects.get_current()
                if contact is None:
                    cart = Cart(site=site)
                else:
                    cart = Cart(site=site, customer=contact)
                cart.save()
                request.session['cart'] = cart.id

            elif return_nullcart:
                cart = NullCart()

            else:
                raise Cart.DoesNotExist()

        #log.debug("Cart: %s", cart)
        return cart


class Cart(models.Model):
    """
    Store items currently in a cart
    The desc isn't used but it is needed to make the admin interface work appropriately
    Could be used for debugging
    """
    site = models.ForeignKey(Site, verbose_name=_('Site'))
    desc = models.CharField(_("Description"), blank=True, null=True, max_length=10)
    date_time_created = models.DateTimeField(_("Creation Date"))
    customer = models.ForeignKey(Contact, blank=True, null=True, verbose_name=_('Customer'))

    objects = CartManager()

    def _get_count(self):
        itemCount = 0
        for item in self.cartitem_set.all():
            itemCount += item.quantity
        return (itemCount)
    numItems = property(_get_count)

    def _get_discount(self):
        return self.undiscounted_total - self.total

    discount = property(_get_discount)

    def _get_total(self, include_discount=True):
        total = Decimal("0")
        for item in self.cartitem_set.all():
            if include_discount:
                total += item.line_total
            else:
                total += item.undiscounted_line_total
        return(total)
    total = property(_get_total)

    def _get_undiscounted_total(self):
        return self._get_total(False)

    undiscounted_total = property(_get_undiscounted_total)

    def __iter__(self):
        return iter(self.cartitem_set.all())

    def __len__(self):
        return self.cartitem_set.count()

    def __nonzero__(self):
        """
        This is used by django to evaluate whether or not to
        include an object when dumping data. Therefore, we will dump
        carts even if they have no items. This is ok because the most likely
        scenario is moving data from one db to the next. See ticket #1015 for
        discussion.

        Use len(cart) if you want to know if there are items in the cart.
        """
        return True

    def _is_empty(self):
        return self.cartitem_set.count() == 0
    is_empty = property(_is_empty)

    def __unicode__(self):
        return u"Shopping Cart (%s)" % self.date_time_created

    def add_item(self, chosen_item, number_added, details=[]):
        alreadyInCart = False
        # Custom Products will not be added, they will each get their own line item
        if 'CustomProduct' in chosen_item.get_subtypes():
            item_to_modify = CartItem(cart=self, product=chosen_item, quantity=Decimal('0'))
        else:
            item_to_modify = CartItem(cart=self, product=chosen_item, quantity=Decimal('0'))
            for similarItem in self.cartitem_set.filter(product__id = chosen_item.id):
                looksTheSame = len(details) == similarItem.details.count()
                if looksTheSame:
                    for detail in details:
                        try:
                            similarItem.details.get(
                                    name=detail['name'],
                                    value=str(detail['value']),     # typecasting for Postgresql
                                    price_change=detail['price_change']
                                    )
                        except CartItemDetails.DoesNotExist:
                            looksTheSame = False
                if looksTheSame:
                    item_to_modify = similarItem
                    alreadyInCart = True
                    break

        # Verify that the 'item_to_modify' can be added to the cart regardless
        # of whether or not it is already in the cart
        signals.satchmo_cart_add_verify.send(
            self,
            cart=self,
            cartitem=item_to_modify,
            added_quantity=number_added,
            details=details)

        if not alreadyInCart:
            self.cartitem_set.add(item_to_modify)

        item_to_modify.quantity += number_added
        item_to_modify.save()
        if not alreadyInCart:
            for data in details:
                item_to_modify.add_detail(data)

        return item_to_modify

    def remove_item(self, chosen_item_id, number_removed):
        item_to_modify = self.cartitem_set.get(id = chosen_item_id)
        item_to_modify.quantity -= number_removed
        if item_to_modify.quantity <= 0:
            item_to_modify.delete()
        self.save()


    def merge_carts(self, src_cart):
        """
        Merge the items from the src_cart into
        the destination. Source cart will be emptied
        """
        for item in src_cart.cartitem_set.all():
            self.add_item(item.product, item.quantity, item.details.all())
            item.delete()

    def empty(self):
        for item in self.cartitem_set.all():
            item.delete()
        self.save()

    def save(self, **kwargs):
        """Ensure we have a date_time_created before saving the first time."""
        if not self.pk:
            self.date_time_created = datetime.datetime.now()
        try:
            site = self.site
        except Site.DoesNotExist:
            self.site = Site.objects.get_current()
        super(Cart, self).save(**kwargs)

    def _get_shippable(self):
        """Return whether the cart contains shippable items."""
        for cartitem in self.cartitem_set.all():
            if cartitem.is_shippable:
                return True
        return False
    is_shippable = property(_get_shippable)

    def get_shipment_list(self):
        """Return a list of shippable products, where each item is split into
        multiple elements, one for each quantity."""
        items = []
        for cartitem in self.cartitem_set.all():
            if cartitem.is_shippable:
                p = cartitem.product
                q =  int(cartitem.quantity.quantize(Decimal('0'), ROUND_CEILING))
                for single in range(0, q):
                    items.append(p)
        return items

    class Meta:
        verbose_name = _("Shopping Cart")
        verbose_name_plural = _("Shopping Carts")

class NullCartItem(object):
    def __init__(self, itemid):
        self.id = itemid
        self.quantity = Decimal('0')
        self.line_total = 0

class CartItem(models.Model):
    """
    An individual item in the cart
    """
    cart = models.ForeignKey(Cart, verbose_name=_('Cart'))
    product = models.ForeignKey(Product, verbose_name=_('Product'))
    quantity = models.DecimalField(_("Quantity"),  max_digits=18,  decimal_places=6)

    def _get_line_unitprice(self, include_discount=True):
        # Get the qty discount price as the unit price for the line.

        if config_value('SHOP','CART_QTY'):
            qty = self.cart.numItems
        else:
            qty = self.quantity

        self.qty_price = self.get_qty_price(qty, include_discount=include_discount)

        self.detail_price = self.get_detail_price()
        #send signal to possibly adjust the unitprice
        if include_discount:
            signals.satchmo_cartitem_price_query.send(self, cartitem=self)
        price = self.qty_price + self.detail_price

        #clean up temp vars
        del self.qty_price
        del self.detail_price

        return price

    unit_price = property(_get_line_unitprice)

    def _get_undiscounted_unitprice(self):
        return self._get_line_unitprice(include_discount=False)

    undiscounted_unit_price = property(_get_undiscounted_unitprice)

    def get_detail_price(self):
        """Get the delta price based on detail modifications"""
        delta = Decimal("0")
        if self.has_details:
            for detail in self.details.all():
                if detail.price_change and detail.value:
                    delta += detail.price_change
        return delta

    def get_qty_price(self, qty, include_discount=True):
        """Get the price for for each unit before any detail modifications"""
        return self.product.get_qty_price(qty, include_discount=include_discount)

    def _get_line_total(self):
        return self.unit_price * self.quantity
    line_total = property(_get_line_total)

    def _get_undiscounted_line_total(self):
        return self.undiscounted_unit_price * self.quantity

    undiscounted_line_total = property(_get_undiscounted_line_total)

    def _get_description(self):
        return self.product.translated_name()
    description = property(_get_description)

    def _is_shippable(self):
        return self.product.is_shippable

    is_shippable = property(fget=_is_shippable)

    def add_detail(self, data):
        detl = CartItemDetails(
            cartitem=self,
            name=data['name'],
            value=data['value'],
            sort_order=data['sort_order'],
            price_change=data['price_change'])

        detl.save()
        #self.details.add(detl)

    def _has_details(self):
        """
        Determine if this specific item has more detail
        """
        return (self.details.count() > 0)

    has_details = property(_has_details)

    def __unicode__(self):
        money_format = force_unicode(moneyfmt(self.line_total))
        return u'%s - %s %s' % (self.quantity, self.product.name,
            money_format)

    class Meta:
        verbose_name = _("Cart Item")
        verbose_name_plural = _("Cart Items")
        ordering = ('id',)

class CartItemDetails(models.Model):
    """
    An arbitrary detail about a cart item.
    """
    cartitem = models.ForeignKey(CartItem, related_name='details', )
    value = models.TextField(_('detail'))
    name = models.CharField(_('name'), max_length=100)
    price_change = CurrencyField(_("Item Detail Price Change"), max_digits=6,
        decimal_places=2, blank=True, null=True)
    sort_order = models.IntegerField(_("Sort Order"),
        help_text=_("The display order for this group."))

    class Meta:
        ordering = ('sort_order',)
        verbose_name = _("Cart Item Detail")
        verbose_name_plural = _("Cart Item Details")


ORDER_CHOICES = (
    ('Online', _('Online')),
    ('In Person', _('In Person')),
    ('Show', _('Show')),
)

ORDER_STATUS = (
    ('Temp', _('Temp')),
    ('New', _('New')),
    ('Blocked', _('Blocked')),
    ('In Process', _('In Process')),
    ('Billed', _('Billed')),
    ('Shipped', _('Shipped')),
    ('Complete', _('Complete')),
    ('Cancelled', _('Cancelled')),
)

class OrderManager(models.Manager):
    def from_request(self, request):
        """Get the order from the session

        Returns:
        - Order object
        """
        order = None
        if 'orderID' in request.session:
            try:
                order = Order.objects.get(id=request.session['orderID'])
                # TODO: Validate against logged-in user.
            except Order.DoesNotExist:
                pass

            if not order:
                del request.session['orderID']

        if not order:
            raise Order.DoesNotExist()

        return order

    def remove_partial_order(self, request):
        """Delete cart from request if it exists and is incomplete (has no status)"""
        try:
            order = Order.objects.from_request(request)
            if not order.status:
                del request.session['orderID']
                log.info("Deleting incomplete order #%i from database", order.id)
                order.delete()
                return True
        except Order.DoesNotExist:
            pass
        return False

class Order(models.Model):
    """
    Orders contain a copy of all the information at the time the order was
    placed.
    """
    site = models.ForeignKey(Site, verbose_name=_('Site'))
    contact = models.ForeignKey(Contact, verbose_name=_('Contact'))
    ship_addressee = models.CharField(_("Addressee"), max_length=61, blank=True)
    ship_street1 = models.CharField(_("Street"), max_length=80, blank=True)
    ship_street2 = models.CharField(_("Street"), max_length=80, blank=True)
    ship_city = models.CharField(_("City"), max_length=50, blank=True)
    ship_state = models.CharField(_("State"), max_length=50, blank=True)
    ship_postal_code = models.CharField(_("Zip Code"), max_length=30, blank=True)
    ship_country = models.CharField(_("Country"), max_length=2, blank=True)
    bill_addressee = models.CharField(_("Addressee"), max_length=61, blank=True)
    bill_street1 = models.CharField(_("Street"), max_length=80, blank=True)
    bill_street2 = models.CharField(_("Street"), max_length=80, blank=True)
    bill_city = models.CharField(_("City"), max_length=50, blank=True)
    bill_state = models.CharField(_("State"), max_length=50, blank=True)
    bill_postal_code = models.CharField(_("Zip Code"), max_length=30, blank=True)
    bill_country = models.CharField(_("Country"), max_length=2, blank=True)
    notes = models.TextField(_("Notes"), blank=True, null=True)
    sub_total = CurrencyField(_("Subtotal"),
        max_digits=18, decimal_places=10, blank=True, null=True, display_decimal=4)
    total = CurrencyField(_("Total"),
        max_digits=18, decimal_places=10, blank=True, null=True, display_decimal=4)
    discount_code = models.CharField(
        _("Discount Code"), max_length=20, blank=True, null=True,
        help_text=_("Coupon Code"))
    discount = CurrencyField(_("Discount amount"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    method = models.CharField(_("Order method"),
        choices=ORDER_CHOICES, max_length=50, blank=True)
    shipping_description = models.CharField(_("Shipping Description"),
        max_length=50, blank=True, null=True)
    shipping_method = models.CharField(_("Shipping Method"),
        max_length=50, blank=True, null=True)
    shipping_model = ShippingChoiceCharField(_("Shipping Models"),
        max_length=30, blank=True, null=True)
    shipping_cost = CurrencyField(_("Shipping Cost"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    shipping_discount = CurrencyField(_("Shipping Discount"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    tax = CurrencyField(_("Tax"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    time_stamp = models.DateTimeField(_("Timestamp"), blank=True, null=True)
    status = models.CharField(_("Status"), max_length=20, choices=ORDER_STATUS,
        blank=True, help_text=_("This is set automatically."))

    objects = OrderManager()

    def __unicode__(self):
        return "Order #%s: %s" % (self.id, self.contact.full_name)

    def add_status(self, status=None, notes=""):
        orderstatus = OrderStatus()
        if not status:
            try:
                curr_status = self.orderstatus_set.latest()
                status = curr_status.status
            except OrderStatus.DoesNotExist:
                status = 'New'

        orderstatus.status = status
        orderstatus.notes = notes
        orderstatus.order = self
        orderstatus.save()

    def add_variable(self, key, value):
        """Add an OrderVariable, used for misc stuff that is just too small to get its own field"""
        try:
            v = self.variables.get(key__exact=key)
            v.value = value
        except OrderVariable.DoesNotExist:
            v = OrderVariable(order=self, key=key, value=value)
        v.save()

    def _authorized_remaining(self):
        auths = [p.amount for p in self.authorizations.filter(complete=False)]
        if auths:
            amount = reduce(operator.add, auths)
        else:
            amount = Decimal('0.00')

        return amount

    authorized_remaining = property(fget=_authorized_remaining)

    def _get_count(self):
        itemCount = 0
        for item in self.orderitem_set.all():
            itemCount += item.quantity
        return (itemCount)
    numItems = property(_get_count)

    def get_variable(self, key, default=None):
        qry = self.variables.filter(key__exact=key)
        ct = qry.count()
        if ct == 0:
            return default
        else:
            return qry[0]

    def copy_addresses(self):
        """
        Copy the addresses so we know what the information was at time of order.
        """
        shipaddress = self.contact.shipping_address
        billaddress = self.contact.billing_address
        self.ship_addressee = shipaddress.addressee
        self.ship_street1 = shipaddress.street1
        self.ship_street2 = shipaddress.street2
        self.ship_city = shipaddress.city
        self.ship_state = shipaddress.state
        self.ship_postal_code = shipaddress.postal_code
        self.ship_country = shipaddress.country.iso2_code
        self.bill_addressee = billaddress.addressee
        self.bill_street1 = billaddress.street1
        self.bill_street2 = billaddress.street2
        self.bill_city = billaddress.city
        self.bill_state = billaddress.state
        self.bill_postal_code = billaddress.postal_code
        self.bill_country = billaddress.country.iso2_code

    def remove_all_items(self):
        """Delete all items belonging to this order."""
        for item in self.orderitem_set.all():
            item.delete()
        self.save()

    def _balance(self):
        if self.total is None:
            self.force_recalculate_total(save=True)
        return trunc_decimal(self.total-self.balance_paid, 2)

    balance = property(fget=_balance)

    def balance_forward(self):
        return moneyfmt(self.balance)

    balance_forward = property(fget=balance_forward)

    def _balance_paid(self):
        payments = [p.amount for p in self.payments.all()]
        if payments:
            paid = reduce(operator.add, payments)
        else:
            paid = Decimal("0.0000000000")

        return paid + self.authorized_remaining

    balance_paid = property(_balance_paid)

    def _credit_card(self):
        """Return the credit card associated with this payment."""
        from payment.models import CreditCardDetail
        for payment in self.payments.order_by('-time_stamp'):
            try:
                if payment.creditcards.count() > 0:
                    return payment.creditcards.get()
            except CreditCardDetail.DoesNotExist:
                pass
        return None
    credit_card = property(_credit_card)

    def _full_bill_street(self, delim="\n"):
        """
        Return both billing street entries separated by delim.
        Note - Use linebreaksbr filter to convert to html in templates.
        """
        if self.bill_street2:
            address = self.bill_street1 + delim + self.bill_street2
        else:
            address = self.bill_street1
        return mark_safe(address)
    full_bill_street = property(_full_bill_street)

    def _full_ship_street(self, delim="\n"):
        """
        Return both shipping street entries separated by delim.
        Note - Use linebreaksbr filterto convert to html in templates.
        """
        if self.ship_street2:
            address = self.ship_street1 + delim + self.ship_street2
        else:
            address = self.ship_street1
        return mark_safe(address)
    full_ship_street = property(_full_ship_street)

    def _ship_country_name(self):
        return Country.objects.get(iso2_code=self.ship_country).name
    ship_country_name = property(_ship_country_name)

    def _bill_country_name(self):
        return Country.objects.get(iso2_code=self.bill_country).name
    bill_country_name = property(_bill_country_name)

    def _discounted_sub_total(self):
        return self.sub_total - self.item_discount

    discounted_sub_total = property(_discounted_sub_total)

    def _get_balance_remaining_url(self):
        return ('satchmo_balance_remaining_order', None, {'order_id' : self.id})
    get_balance_remaining_url = models.permalink(_get_balance_remaining_url)

    def _partially_paid(self):
        return self.balance_paid > Decimal("0.0000000000")

    partially_paid = property(_partially_paid)

    def _is_partially_paid(self):
        if self.total:
            return (
                float(self.balance) > 0.0
                and float(self.balance_paid) > 0.0
                and self.balance != self.balance_paid
                )
        else:
            return False

    is_partially_paid = property(fget=_is_partially_paid)

    def payments_completed(self):
        q = self.payments.exclude(transaction_id__isnull = False, transaction_id = "PENDING")
        result = [p for p in q if p.amount]
        return result

    def save(self, **kwargs):
        """
        Copy addresses from contact. If the order has just been created, set
        the create_date.
        """
        if not self.pk:
            self.time_stamp = datetime.datetime.now()
            self.copy_addresses()
        super(Order, self).save(**kwargs) # Call the "real" save() method.

    def invoice(self):
        url = urlresolvers.reverse('satchmo_print_shipping', None, None, {'doc' : 'invoice', 'id' : self.id})
        return mark_safe(u'<a href="%s">%s</a>' % (url, ugettext('View')))
    invoice.allow_tags = True

    def _item_discount(self):
        """Get the discount of just the items, less the shipping discount."""
        return self.discount-self.shipping_discount
    item_discount = property(_item_discount)

    def packingslip(self):
        url = urlresolvers.reverse('satchmo_print_shipping', None, None, {'doc' : 'packingslip', 'id' : self.id})
        return mark_safe(u'<a href="%s">%s</a>' % (url, ugettext('View')))
    packingslip.allow_tags = True

    def recalculate_total(self, save=True):
        """Calculates sub_total, taxes and total if the order is not already partially paid."""
        if self.is_partially_paid:
            log.debug("Order %i - skipping recalculate_total since product is partially paid.", self.id)
        else:
            self.force_recalculate_total(save=save)

    def force_recalculate_total(self, save=True):
        """Calculates sub_total, taxes and total."""
        zero = Decimal("0.0000000000")
        total_discount = Decimal("0.0000000000")

        discount = Discount.objects.by_code(self.discount_code)
        discount.calc(self)

        discounts = discount.item_discounts
        itemprices = []
        fullprices = []

        qty_override = config_value('SHOP','CART_QTY')
        if qty_override:
            itemct = self.numItems

        for lineitem in self.orderitem_set.all():
            lid = lineitem.id
            if lid in discounts:
                lineitem.discount = discounts[lid]
                total_discount += lineitem.discount
                #log.debug('total_discount (calc): %s', total_discount)
            else:
                lineitem.discount = zero
            # now double check against other discounts, such as tiered discounts
            if qty_override:
                qty = itemct
            else:
                qty = lineitem.quantity

            adjustment = get_product_quantity_adjustments(lineitem.product, qty=qty)

            if adjustment and adjustment.price:
                baseprice = adjustment.price.price
                finalprice = adjustment.final_price()
                #We need to add in any OrderItemDetail price adjustments before we do anything else
                baseprice += lineitem.get_detail_price()
                finalprice += lineitem.get_detail_price()
                if baseprice > finalprice or baseprice != lineitem.unit_price:
                    unitdiscount = (lineitem.discount/lineitem.quantity) + baseprice-finalprice
                    unitdiscount = trunc_decimal(unitdiscount, 2)
                    linediscount = unitdiscount * lineitem.quantity
                    total_discount += linediscount
                    #log.debug('total_discount (line): %s', total_discount)
                    fullydiscounted = (baseprice - unitdiscount) * lineitem.quantity
                    lineitem.unit_price = baseprice
                    lineitem.discount = linediscount
                    lineitem.line_item_price = baseprice * lineitem.quantity
                    log.debug('Adjusting lineitem unit price for %s. Full price=%s, discount=%s.  Final price for qty %d is %s',
                        lineitem.product.slug, baseprice, unitdiscount, lineitem.quantity, fullydiscounted)
            if save:
                lineitem.save()

            itemprices.append(lineitem.sub_total)
            fullprices.append(lineitem.line_item_price)

        shipprice = Price()
        shipprice.price = self.shipping_cost
        shipadjust = PriceAdjustmentCalc(shipprice)
        if 'Shipping' in discounts:
            shipadjust += PriceAdjustment('discount', _('Discount'), discounts['Shipping'])

        signals.satchmo_shipping_price_query.send(self, adjustment=shipadjust)
        shipdiscount = shipadjust.total_adjustment()
        self.shipping_discount = shipdiscount
        total_discount += shipdiscount
        #log.debug('total_discount (+ship): %s', total_discount)

        self.discount = total_discount

        if itemprices:
            item_sub_total = reduce(operator.add, itemprices)
        else:
            item_sub_total = zero

        if fullprices:
            full_sub_total = reduce(operator.add, fullprices)
        else:
            full_sub_total = zero

        self.sub_total = full_sub_total

        taxProcessor = get_tax_processor(self)
        totaltax, taxrates = taxProcessor.process()
        self.tax = totaltax

        # clear old taxes
        for taxdetl in self.taxes.all():
            taxdetl.delete()

        for taxdesc, taxamt in taxrates.items():
            taxdetl = OrderTaxDetail(order=self, tax=taxamt, description=taxdesc, method=taxProcessor.method)
            taxdetl.save()

        log.debug("Order #%i, recalc: sub_total=%s, shipping=%s, discount=%s, tax=%s",
            self.id,
            moneyfmt(item_sub_total),
            moneyfmt(self.shipping_sub_total),
            moneyfmt(self.discount),
            moneyfmt(self.tax))

        self.total = Decimal(item_sub_total + self.shipping_sub_total + self.tax)

        if save:
            self.save()

    def shippinglabel(self):
        url = urlresolvers.reverse('satchmo_print_shipping', None, None, {'doc' : 'shippinglabel', 'id' : self.id})
        return mark_safe(u'<a href="%s">%s</a>' % (url, ugettext('View')))
    shippinglabel.allow_tags = True

    def _order_total(self):
        #Needed for the admin list display
        return moneyfmt(self.total)
    order_total = property(_order_total)

    def order_success(self):
        """Run each item's order_success method."""
        log.info("Order success: %s", self)
        for orderitem in self.orderitem_set.all():
            subtype = orderitem.product.get_subtype_with_attr('order_success')
            if subtype:
                subtype.order_success(self, orderitem)

        signals.order_success.send(self, order=self)

    def order_cancel(self):
        """Ask if the order can be cancelled. By default, do not cancel shipped, completed and
        already cancelled orders."""
        self.is_cancellable = self.status not in ('Shipped', 'Completed', 'Cancelled')
        # listeners can override default flag setting and (dis)allow cancellation
        signals.order_cancel_query.send(self, order=self)
        if self.is_cancellable:
           self.add_status('Cancelled')
           signals.order_cancelled.send(self, order=self)
        return self.is_cancellable

    def _paid_in_full(self):
        """True if total has been paid"""
        return self.balance == Decimal('0.00')
    paid_in_full = property(fget=_paid_in_full)

    def _has_downloads(self):
        """Determine if there are any downloadable products on this order"""
        if 'product.modules.downloadable' in settings.INSTALLED_APPS \
            and self.downloadlink_set.count() > 0:
            return True
        return False
    has_downloads = property(_has_downloads)

    def _is_downloadable(self):
        """Determine if all products on this order are downloadable"""
        for orderitem in self.orderitem_set.all():
           if not orderitem.product.is_downloadable:
               return False
        return True
    is_downloadable = property(_is_downloadable)

    def _is_shippable(self):
        """Determine if we will be shipping any items on this order """
        for orderitem in self.orderitem_set.all():
            if orderitem.is_shippable:
                return True
        return False
    is_shippable = property(_is_shippable)

    def _shipping_sub_total(self):
        if self.shipping_cost is None:
            self.shipping_cost = Decimal('0.00')
        if self.shipping_discount is None:
            self.shipping_discount = Decimal('0.00')
        return self.shipping_cost-self.shipping_discount
    shipping_sub_total = property(_shipping_sub_total)

    def _shipping_tax(self):
        rates = self.taxes.filter(description__iexact = 'shipping')
        if rates.count()>0:
            tax = reduce(operator.add, [t.tax for t in rates])
        else:
            tax = Decimal("0.0000000000")
        return tax
    shipping_tax = property(_shipping_tax)

    def _shipping_with_tax(self):
        return self.shipping_sub_total + self.shipping_tax
    shipping_with_tax = property(_shipping_with_tax)

    def sub_total_with_tax(self):
        return reduce(operator.add, [o.total_with_tax for o in self.orderitem_set.all()])

    def update_status(self, status):
        """WARNING: To just change order status, use Order.add_status().
        This method is called back when OrderStatus is saved and does not create required object."""
        oldstatus = self.status
        self.status = status
        self.save()
        if (oldstatus != self.status):
            signals.satchmo_order_status_changed.send(self, oldstatus=oldstatus, newstatus=status, order=self)

    def validate(self, request):
        """
        Return whether the order is valid.
        Not guaranteed to be side-effect free.
        """
        valid = True
        for orderitem in self.orderitem_set.all():
            for subtype_name in orderitem.product.get_subtypes():
                subtype = getattr(orderitem.product, subtype_name.lower())
                validate_method = getattr(subtype, 'validate_order', None)
                if validate_method:
                    valid = valid and validate_method(request, self, orderitem)
        return valid

    class Meta:
        verbose_name = _("Product Order")
        verbose_name_plural = _("Product Orders")

class OrderItem(models.Model):
    """
    A line item on an order.
    """
    order = models.ForeignKey(Order, verbose_name=_("Order"))
    product = models.ForeignKey(Product, verbose_name=_("Product"))
    quantity = models.DecimalField(_("Quantity"),  max_digits=18,  decimal_places=6)
    unit_price = CurrencyField(_("Unit price"),
        max_digits=18, decimal_places=10)
    unit_tax = CurrencyField(_("Unit tax"), default=Decimal('0.00'),
        max_digits=18, decimal_places=10)
    line_item_price = CurrencyField(_("Line item price"),
        max_digits=18, decimal_places=10)
    tax = CurrencyField(_("Line item tax"), default=Decimal('0.00'),
        max_digits=18, decimal_places=10)
    expire_date = models.DateField(_("Subscription End"), help_text=_("Subscription expiration date."), blank=True, null=True)
    completed = models.BooleanField(_("Completed"), default=False)
    discount = CurrencyField(_("Line item discount"),
        max_digits=18, decimal_places=10, blank=True, null=True)

    def __unicode__(self):
        return self.product.translated_name()

    def _get_category(self):
        return(self.product.get_category.translated_name())
    category = property(_get_category)

    def _is_shippable(self):
        return self.product.is_shippable

    is_shippable = property(fget=_is_shippable)

    def _has_details(self):
        """Determine if this specific item has more detail"""
        return (self.orderitemdetail_set.count() > 0)

    has_details = property(_has_details)

    def get_detail_price(self):
        """Get the delta price based on detail modifications"""
        delta = Decimal("0.000000")
        if self.has_details:
            for detail in self.orderitemdetail_set.all():
                if detail.price_change and detail.value:
                    delta += detail.price_change
        return delta

    def _sub_total(self):
        if self.discount:
            return self.line_item_price-self.discount
        else:
            return self.line_item_price
    sub_total = property(_sub_total)

    def _total_with_tax(self):
        return self.sub_total + self.tax
    total_with_tax = property(_total_with_tax)

    def _unit_price_with_tax(self):
        return self.unit_price + self.unit_tax
    unit_price_with_tax = property(_unit_price_with_tax)

    def _get_description(self):
        return self.product.translated_name()
    description = property(_get_description)

    def _get_line_total(self):
        return self.unit_price * self.quantity
    line_total = property(_get_line_total)

    def save(self, **kwargs):
        self.update_tax()
        super(OrderItem, self).save(**kwargs)

    def update_tax(self):
        taxclass = self.product.taxClass
        processor = get_tax_processor(order=self.order)

        if self.product.taxable:
            self.unit_tax = processor.by_price(taxclass, self.unit_price)
            self.tax = processor.by_orderitem(self)

    class Meta:
        verbose_name = _("Order Line Item")
        verbose_name_plural = _("Order Line Items")
        ordering = ('id',)

class OrderItemDetail(models.Model):
    """
    Name, value pair and price delta associated with a specific item in an order
    """
    item = models.ForeignKey(OrderItem, verbose_name=_("Order Item"), )
    name = models.CharField(_('Name'), max_length=100)
    value = models.CharField(_('Value'), max_length=255)
    price_change = CurrencyField(_("Price Change"), max_digits=18, decimal_places=10, blank=True, null=True)
    sort_order = models.IntegerField(_("Sort Order"),
        help_text=_("The display order for this group."))

    def __unicode__(self):
        return u"%s - %s,%s" % (self.item, self.name, self.value)

    class Meta:
        verbose_name = _("Order Item Detail")
        verbose_name_plural = _("Order Item Details")
        ordering = ('sort_order',)

class OrderStatus(models.Model):
    """
    An order will have multiple statuses as it moves its way through processing.
    """
    order = models.ForeignKey(Order, verbose_name=_("Order"))
    status = models.CharField(_("Status"),
        max_length=20, choices=ORDER_STATUS, blank=True)
    notes = models.CharField(_("Notes"), max_length=100, blank=True)
    time_stamp = models.DateTimeField(_("Timestamp"))

    def __unicode__(self):
        return self.status

    def save(self, **kwargs):
        if not self.pk and not self.time_stamp:
            self.time_stamp = datetime.datetime.now()
        super(OrderStatus, self).save(**kwargs)
        self.order.update_status(self.status)

    class Meta:
        verbose_name = _("Order Status")
        verbose_name_plural = _("Order Statuses")
        ordering = ('time_stamp',)
        get_latest_by = 'time_stamp'

class OrderPaymentBase(models.Model):
    payment = PaymentChoiceCharField(_("Payment Method"),
        max_length=25, blank=True)
    amount = CurrencyField(_("amount"),
        max_digits=18, decimal_places=10, blank=True, null=True)
    time_stamp = models.DateTimeField(_("timestamp"), blank=True, null=True)
    transaction_id = models.CharField(_("Transaction ID"), max_length=45, blank=True, null=True)
    details = models.CharField(_("Details"), max_length=255, blank=True, null=True)
    reason_code = models.CharField(_('Reason Code'),  max_length=255, blank=True, null=True)

    def _credit_card(self):
        """Return the credit card associated with this payment."""
        try:
            return self.creditcards.get()
        except self.creditcards.model.DoesNotExist:
            return None
    credit_card = property(_credit_card)

    def _amount_total(self):
        return moneyfmt(self.amount)

    amount_total = property(fget=_amount_total)

    def save(self, **kwargs):
        if not self.pk:
            self.time_stamp = datetime.datetime.now()

        super(OrderPaymentBase, self).save(**kwargs)

    class Meta:
        abstract = True

class OrderAuthorization(OrderPaymentBase):
    order = models.ForeignKey(Order, related_name="authorizations")
    capture = models.ForeignKey('OrderPayment', related_name="authorizations")
    complete = models.BooleanField(_('Complete'), default=False)

    def __unicode__(self):
        if self.id is not None:
            return u"Order Authorization #%i" % self.id
        else:
            return u"Order Authorization (unsaved)"

    def remaining(self):
        payments = [p.amount for p in self.order.payments.all()]
        if payments:
            amount = reduce(operator.add, payments)
        else:
            amount = Decimal('0.00')

        remaining = self.order.total - amount
        if remaining > self.amount:
            remaining = self.amount

        return trunc_decimal(remaining, 2)

    def save(self, **kwargs):
        # create linked payment
        try:
            capture = self.capture
        except OrderPayment.DoesNotExist:
            log.debug('Payment Authorization - creating linked payment')
            log.debug('order is: %s', self.order)
            self.capture = OrderPayment.objects.create_linked(self)
        super(OrderPaymentBase, self).save(**kwargs)

    class Meta:
        verbose_name = _("Order Payment Authorization")
        verbose_name_plural = _("Order Payment Authorizations")

class OrderPaymentManager(models.Manager):
    def create_linked(self, other, **kwargs):
        linked = OrderPayment(
                order = other.order,
                payment = other.payment,
                amount=Decimal('0.00'),
                transaction_id="LINKED",
                details=other.details,
                reason_code="")
        linked.save(**kwargs)
        return linked

class OrderPayment(OrderPaymentBase):
    order = models.ForeignKey(Order, related_name="payments")

    objects = OrderPaymentManager()

    def __unicode__(self):
        if self.id is not None:
            return u"Order Payment #%i" % self.id
        else:
            return u"Order Payment (unsaved)"

    class Meta:
        verbose_name = _("Order Payment")
        verbose_name_plural = _("Order Payments")

class OrderPendingPayment(OrderPaymentBase):
    order = models.ForeignKey(Order, related_name="pendingpayments")
    capture = models.ForeignKey('OrderPayment', related_name="pendingpayments")

    def __unicode__(self):
        if self.id is not None:
            return u"Order Pending Payment #%i" % self.id
        else:
            return u"Order Pending Payment (unsaved)"

    def save(self, **kwargs):
        # create linked payment
        try:
            capture = self.capture
        except OrderPayment.DoesNotExist:
            log.debug('Pending Payment - creating linked payment')
            self.capture = OrderPayment.objects.create_linked(self, **kwargs)
        super(OrderPaymentBase, self).save(**kwargs)

    class Meta:
        verbose_name = _("Order Pending Payment")
        verbose_name_plural = _("Order Pending Payments")

class OrderPaymentFailure(OrderPaymentBase):
    order = models.ForeignKey(Order, null=True, blank=True, related_name='paymentfailures')

class OrderVariable(models.Model):
    order = models.ForeignKey(Order, related_name="variables")
    key = models.SlugField(_('key'), )
    value = models.CharField(_('value'), max_length=100)

    class Meta:
        ordering=('key',)
        verbose_name = _("Order variable")
        verbose_name_plural = _("Order variables")

    def __unicode__(self):
        if len(self.value)>10:
            v = self.value[:10] + '...'
        else:
            v = self.value
        return u"OrderVariable: %s=%s" % (self.key, v)

class OrderTaxDetail(models.Model):
    """A tax line item"""
    order = models.ForeignKey(Order, related_name="taxes")
    method = models.CharField(_("Model"), max_length=50, )
    description = models.CharField(_("Description"), max_length=50, blank=True)
    tax = CurrencyField(_("Tax"),
        max_digits=18, decimal_places=10, blank=True, null=True)

    def __unicode__(self):
        if self.description:
            return u"Tax: %s %s" % (self.description, self.tax)
        else:
            return u"Tax: %s" % self.tax

    class Meta:
        verbose_name = _('Order tax detail')
        verbose_name_plural = _('Order tax details')
        ordering = ('id',)

import config

import listeners
listeners.start_default_listening()



#!/usr/bin/env python
# Default model classes for use in BlogEngine

from notmm.dbapi.orm import model

class AuthorManager(model.ModelManager):
    model = 'Author'

class CommentManager(model.ModelManager):
    model = 'Comment'

class MessageManager(model.ModelManager):
    model = 'Message'




'''
FedEx Shipping Module
v0.4
By Neum Schmickrath - www.pageworthy.com

You must have a Fedex account to use this module.
You may register at fedex.com
'''

from decimal import Decimal
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe 
from django.template import loader, Context
from django.core.cache import cache

from shipping.modules.base import BaseShipper
from shipping import signals
from livesettings import config_get_group

import urllib2
from xml.dom import minidom
import logging

log = logging.getLogger('fedex.shipper')

class Shipper(BaseShipper):
    
    def __init__(self, cart=None, contact=None, service_type=None):

        self._calculated = False
        self.cart = cart
        self.contact = contact
        self.raw_results = '(not processed)'

        if service_type:    
            self.service_type_code = service_type[0]
            self.service_type_text = service_type[1]

        else:
            self.service_type_code = '99'
            self.service_type_text = 'Uninitialized'
# Had to edit this so the shipping name did not error out for being more than 30 characters. Old code is commented out.
        #self.id = u'FedEx-%s-%s' % (self.service_type_code, self.service_type_text)
        self.id = u'%s' % (self.service_type_text)

        #if cart or contact:
        #  self.calculate(cart, contact)
  
    def __str__(self):
        '''
          This is mainly helpful for debugging purposes
        '''

        return 'FedEx'


    def __unicode__(self):
        '''
          As is this.
        '''

        return 'FedEx'
    
    def description(self):
        '''
          A basic description that will be displayed to the user when 
          selecting their shipping options
        '''

        return _('FedEx - %s' % self.service_type_text)

    def cost(self):
        '''
          Complex calculations can be done here as long as the return 
          value is a decimal figure
        '''

        assert(self._calculated)
        return(Decimal(self.charges))

    def method(self):
        '''
          Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        '''

        return _('FedEx')

    def expectedDelivery(self):
        '''
          Can be a plain string or complex calcuation 
          returning an actual date
        '''

        if self.delivery_days <> '1':
            return _('%s business days' % self.delivery_days)
        else:
            return _('%s business day' % self.delivery_days)
    
    def valid(self, order=None):
        '''
        Can do complex validation about whether or not this
        option is valid. For example, may check to see if the 
        recipient is in an allowed country or location.
        '''

        return self.is_valid

    def _check_for_error(self, response):
        '''
          Check XML response, see if it indicates an error.
          Expects 'response' to already have been run through
          minidom.parseString()
        '''
    
        if response.getElementsByTagName('Error'):
            # we have an error!
            error_code = response.getElementsByTagName('Error')[0].getElementsByTagName('Code')[0].firstChild.nodeValue
            error_mesg = response.getElementsByTagName('Error')[0].getElementsByTagName('Message')[0].firstChild.nodeValue
            log.info('Fedex Error: %s - Code: %s', error_mesg, error_code)
            return (error_mesg, error_code)

        else:
            # all clear.
            return False

    def _process_request(self, connection, request):
        '''
          Post the data and return the XML response
        '''

        conn = urllib2.Request(url=connection, data=request)
        f = urllib2.urlopen(conn)
        all_results = f.read()
        self.raw_response = all_results
        return(minidom.parseString(all_results))
    
    def calculate(self, cart, contact):
        '''
          Based on the chosen Fedex method, we will do our call(s) 
          to FedEx and see how much it will cost. We will also need 
          to store the results for further parsing and return via the
          methods above.
        '''
        log.debug("Starting fedex calculations")

        from satchmo_store.shop.models import Config
        settings =  config_get_group('shipping.modules.fedex')

        verbose = settings.VERBOSE_LOG.value
            
        self.delivery_days = _('3 - 4') #Default setting for ground delivery
        shop_details = Config.objects.get_current()
        self.packaging = ''


        # FedEx Ground Home Delivery Packaging must be YOURPACKAGING only.
        if self.service_type_code in ('FEDEXGROUND', 'GROUNDHOMEDELIVERY'):
            self.packaging = 'YOURPACKAGING'
        else:
            self.packaging = settings.SHIPPING_PACKAGE.value

        if verbose:
            log.debug('Calculating fedex with type=%s, packaging=%s', self.service_type_code, self.packaging)

        self.is_valid = False
        error = False

        if not settings.ACCOUNT.value:
            log.warn("No fedex account found in settings")
            return
        
        if not settings.METER_NUMBER.value:
            log.warn("No fedex meter number found in settings")
            return

        configuration = {
            'account': settings.ACCOUNT.value,
            'meter': settings.METER_NUMBER.value,
            'packaging': self.packaging,
            'ship_type': self.service_type_code,
            'shop_details':shop_details,
        }

        if settings.LIVE.value:
            connection = settings.CONNECTION.value
        else:
            connection = settings.CONNECTION_TEST.value
            

        self.charges = 0
        
        box_weight_units = "LB"

        # FedEx requires that the price be formatted to 2 decimal points.
        # e.g., 1.00, 10.40, 3.50

        # They also require that the weight be one decimal point. 
        # e.g., 1.0, 2.3, 10.4
        
        if settings.SINGLE_BOX.value:
            if verbose:
                log.debug("Using single-box method for fedex calculations.")
                
            box_price = Decimal("0.00")
            box_weight = Decimal("0.00")
            for product in cart.get_shipment_list():
                box_price += product.unit_price
                if product.weight is None:
                    log.warn("No weight on product (skipping for ship calculations): %s", product)
                else:
                    box_weight += product.weight
                if product.weight_units and product.weight_units != "":
                    box_weight_units = product.weight_units
            
            if box_weight < Decimal("0.1"):
                log.debug("Total box weight too small, defaulting to 0.1")
                box_weight = Decimal("0.1")
                
            shippingdata = {
                'config': configuration,
                'box_price': '%.2f' % box_price,
                'box_weight' : '%.1f' % box_weight,
                'box_weight_units' : box_weight_units.upper(),
                'contact': contact,
                'shipping_address' : shop_details,
                'shipping_phone' : shop_details.phone,
                'shipping_country_code' : shop_details.country.iso2_code
            }
            signals.shipping_data_query.send(Shipper, shipper=self, cart=cart, shippingdata=shippingdata)

            c = Context(shippingdata)
            t = loader.get_template('shipping/fedex/request.xml')
            request = t.render(c)

            try:
                response = self._process_request(connection, request)
                error = self._check_for_error(response)
            
                if verbose:
                    log.debug("Fedex request: %s", request)
                    log.debug("Fedex response: %s", self.raw_response)

                if not error:
                    this_charge = float(response.documentElement.getElementsByTagName('NetCharge')[0].firstChild.nodeValue)
                    this_discount = float(response.documentElement.getElementsByTagName('EffectiveNetDiscount')[0].firstChild.nodeValue)
                    self.delivery_days = response.documentElement.getElementsByTagName('TimeInTransit')[0].firstChild.nodeValue

                    total_cost = this_charge + this_discount
                    self.charges += total_cost
            except urllib2.URLError:
                log.warn("Error opening url: %s", connection)
                error = True
                
        else:
            # process each shippable separately

            # I'm not certain why FedEx implemented their 'Web Service' 
            # this way. However, you can't give FedEx a list of boxes 
            # and get back a list of prices (as you can with UPS). 
            # Each box has to be a completely new transaction - that 
            # is, a separate POST to their server.
            #
            # So, to simulate this functionality, and return a total 
            # price, we have to loop through all of our items, and 
            # pray the customer isn't ordering a thousand boxes of bagels.
            for product in cart.get_shipment_list():
                c = Context({
                  'config': configuration,
                  'box_weight' : '%.1f' % (product.weight or 0.0),
                  'box_weight_units' : product.weight_units and product.weight_units.upper() or 'LB',
                  'box_price' : '%.2f' % product.unit_price,
                  'contact': contact,
                })
    
                t = loader.get_template('shipping/fedex/request.xml')
                request = t.render(c)

                response = self._process_request(connection, request)
                error = self._check_for_error(response)
                
                if verbose:
                    log.debug("Fedex request: %s", request)
                    log.debug("Fedex response: %s", self.raw_response)

                if not error:
                    this_charge = float(response.documentElement.getElementsByTagName('NetCharge')[0].firstChild.nodeValue)
                    this_discount = float(response.documentElement.getElementsByTagName('EffectiveNetDiscount')[0].firstChild.nodeValue)
                    self.delivery_days = response.documentElement.getElementsByTagName('TimeInTransit')[0].firstChild.nodeValue

                    total_cost = this_charge + this_discount
                    self.charges += total_cost
                    
                else:
                    break

        if not error:
            self.charges = str(self.charges)
            self.is_valid = True
            self._calculated = True




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from notmm.controllers.auth     import AuthCookieController
from notmm.controllers.wsgi import WSGIController
from notmm.utils.django_settings import LazySettings


# init the django settings subsystem
settings = LazySettings()

__all__ = ['make_app']

def make_app(controller=WSGIController, enable_oauth=False):
    
    wsgi_app = controller(
        settings=settings,
        app_conf={'django.settings_autoload': True,
                  'logging.disabled': False}
    )
    if enable_oauth:
        from wsgi_oauth2 import client
        google_client = client.GoogleClient(
            settings.OAUTH2_CLIENT_ID,
            access_token=settings.OAUTH2_ACCESS_TOKEN,
            scope=settings.OAUTH2_SCOPE, 
            redirect_url=settings.OAUTH2_REDIRECT_URL)
        
        # see wsgi_oauth2.controller.OAuthController
        wsgi_app = google_client.wsgi_middleware(
            wsgi_app, client=google_client,
            secret=settings.SECRET_KEY, 
            login_path=settings.OAUTH2_LOGIN_URL,
            forbidden_passthrough=settings.OAUTH2_FORBIDDEN_PASSTHROUGH)
    return wsgi_app



import sys
#import mainapp
from notmm.utils.wsgilib import HTTPRequest
from test_support import (
    WSGIControllerTestCase, 
    TestClient, 
    unittest,
    make_app,
    settings,
    ResponseClass
    )

class WSGIAppTestCase(WSGIControllerTestCase):

    def test_render_index(self):
        response = self.client.get('/') # 
        self.assertEqual(response[1], '200 OK')

    def test_post_required_with_schevo_db(self):
        #req = HTTPRequest({})
        response = self.client.post('/test_post', data={})
        print ''.join([item for item in response[0]])
        #self.assertEqual(isinstance(response, ResponseClass), True, type(response))
        self.assertEqual(response[1], '200 OK')




# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk
from gtk import gdk
from kiwi.component import get_utility
from kiwi.utils import gsignal, type_register

from gazpacho.app.bars import bar_manager
from gazpacho.annotator import Annotator, draw_annotations
from gazpacho.commandmanager import command_manager
from gazpacho.interfaces import IGazpachoApp
from gazpacho.popup import Popup
from gazpacho.placeholder import Placeholder
from gazpacho.properties import prop_registry
from gazpacho.referencecontainer import ReferenceContainer
from gazpacho.util import get_parent, get_all_children
from gazpacho.widgetregistry import widget_registry

def load_gadget_from_widget(widget, project):
    """  This function creates a gadget from a widget.
    It recursively creates any children of the original widget.
    This method is mainly used when loading a .glade file with
    gazpacho.loader

    Also makes sure the name of the widget is unique in the project
    parameter and if not, change it for a valid one.

    @return: a new gadget
    @rtype: Gadget
    """

    # widget may not have a name if it is an intermediate internal children
    # For example, the label of a button does not have a name
    if widget.name is None:
        # this is not an important widget but maybe its children are
        if isinstance(widget, gtk.Container):
            for child in get_all_children(widget):
                load_gadget_from_widget(child, project)
        return

    type_name = gobject.type_name(widget)
    adaptor = widget_registry.get_by_name(type_name)
    if adaptor is None:
        print('Warning: could not get the class from widget: %s' % widget)
        return

    gadget = adaptor.load(project.context, widget)
    if not isinstance(gadget, Gadget):
        raise TypeError("WidgetAdaptor.load should always return a Gadget"
                        " not %s" % gadget)
    if not gobject.type_is_a(gadget.widget, adaptor.type):
        raise TypeError("%s is not of the type %s"
                        % (gadget.widget, adaptor.type))
    return gadget

# XXX: Rename to Gadget
class BaseGadget(gobject.GObject):
    """
    This is essentially a wrapper around regular widget with more data and
    functionality needed for Gazpacho.

    It has access to the widget it is associated with, the Project it
    belongs to and the instance of its WidgetAdaptor.

    Given a widget you can ask which Gadget is associated with it
    using the get_from_widget function or, if no one has been created yet you
    can create one with the create_from_widget method.

    A gadget has a signal dictionary where the keys are signal names and the
    values are lists of signal handler names. There are methods to
    add/remove/edit these.

    It store all the properties (wrappers for gobject properties) and you can
    get any of them with get_prop. It knows how to handle normal
    properties and packing properties.

    @ivar internal_name: name of internal widget or None
    @ivar adaptor: widget adaptor we belong to
    @ivar project: project tied to gadget
    @ivar widget: widget associated with this gadget
    @ivar constructor: cConstructor, used by UIManager
    @ivar maintain_gtk_properties:
      flag that tells the property code if this widget should keep the
      values of its properties when creating the GazpachoProperties or not
    @ivar references: a reference container for keeping track of which
      objects are refering to this gadget
    """
    gsignal('add-signal', object)
    gsignal('remove-signal', object)

    def __init__(self, adaptor, project):
        """This init method is not complete because there are several scenarios
        where widgets are created:

            - Normal creation in Gazpacho
            - When loading from a .glade file
            - When copying from another gadget
            - Internal gadgets

        So you should call the appropiate method after this basic initialization.
        """
        gobject.GObject.__init__(self)
        self._signals = {} # name -> handlers
        self._deleted_data = None
        self._deleted = False
        self.properties = {}
        self.child_properties = {}

        self.adaptor = adaptor
        self.constructor = None
        self.maintain_gtk_properties = False
        self.internal_name = None
        self.project = project
        # XXX: Rename
        self.widget = None

        self.references = ReferenceContainer(self)

    # XXX: Generalize name

    def __repr__(self):
        if self.widget:
            name = self.widget.name
        else:
            name = '(empty)'
        return '<%s %s>' % (self.__class__.__name__, name)

    # Properties

    def _get_name(self):
        return self.widget.name

    def _set_name(self, value):
        # set_name calls notify::name
        self.widget.set_name(value)

    name = property(_get_name, _set_name)


    def _get_deleted(self):
        return self._deleted

    def _set_deleted(self, deleted):
        if self._deleted == deleted:
            return
        
        self._deleted = deleted

        if not deleted:
            self.adaptor.restore(self.project.context, self.widget, self._deleted_data)
            self._deleted_data = None
            self.references.restore_references()

        # We need to update all children as well
        for child_widget in self.get_children():
            child_gadget = Gadget.from_widget(child_widget)
            if child_gadget:
                child_gadget.deleted = deleted

        if deleted:
            self.references.clear_references()
            self._deleted_data = self.adaptor.delete(self.project.context, self.widget)

    deleted = property(_get_deleted, _set_deleted)

    # Class methods

    #@classmethod
    def load(cls, widget, project):
        """Helper function of load_gadget_from_widget. Create the Gazpacho
        widget and create a right name for it
        """
        assert project != None

        adaptor = widget_registry.get_by_type(widget)
        assert adaptor != None, widget

        gadget = cls(adaptor, project)
        gadget.setup_widget(widget)

        gadget.internal_name = widget.get_data(
            'gazpacho::internal-child-name')

        return gadget
    load = classmethod(load)

    #@classmethod
    def from_xml(cls, project, xml, name=None):
        """
        Creates a new gadget from xml.

        @param project: project
        @param xml: xml string
        @param name: name of widget or None
        """

        from gazpacho.project import GazpachoObjectBuilder
        wt = GazpachoObjectBuilder(buffer=xml, placeholder=Placeholder,
                                   app=project.get_app())
        project.uim.load(wt)

        # Load models before widgets
        models = [w for w in wt.toplevels if isinstance(w, gtk.ListStore)]
        for model in models:
            project.model_manager.load_model(model)

        if name:
            widget = wt.get_widget(name)
        else:
            widgets = wt.get_widgets()
            if len(widgets) > 1:
                raise AssertionError(
                    "should only be one widget: found %r" % widgets)
            widget = widgets[0]
        return load_gadget_from_widget(widget, project)
    from_xml = classmethod(from_xml)

    #@classmethod
    def replace(cls, old_widget, new_widget, parent):
        # FIXME: MESS MESS MESS
        new_gadget = cls.from_widget(new_widget)
        old_gadget = old_widget and cls.from_widget(old_widget)

        new_widget = new_gadget and new_gadget.widget or new_widget
        old_widget = old_gadget and old_gadget.widget or old_widget

        if parent is None:
            parent = get_parent(old_widget)

        parent.adaptor.replace_child(parent.project.context,
                                     old_widget,
                                     new_widget,
                                     parent.widget)
        return new_gadget
    replace = classmethod(replace)

    #@classmethod
    def from_widget(cls, widget):
        return widget.get_data('gazpacho::gadget')
    from_widget = classmethod(from_widget)

    # Public

    def get_children(self):
        """
        Get a list of children for the gadget.
        Note that an adaptor can override the behavior so this
        should be used instead of widget.get_children() directly
        @returns: list of children
        """
        return self.adaptor.get_children(self.project.context,
                                         self.widget)

    def get_prop(self, prop_name):
        return self._internal_get_prop(prop_name, child=False)

    def get_child_prop(self, prop_name):
        return self._internal_get_prop(prop_name, child=True)

    def setup_internal_widget(self, widget, internal_name, parent_name):
        self.internal_name = internal_name
        widget.set_name(parent_name + '-' + internal_name)

        self.setup_widget(widget)

    def setup_widget(self, widget):
        if self.widget is not None:
            self.widget.set_data('gazpacho::gadget', None)
            self.widget = None

        self.widget = widget
        self.constructor = widget.get_data('gazpacho::uimanager-name')
        self.widget.set_data('gazpacho::gadget', self)

    def create_widget(self, interactive=True):
        """Second part of the init process when creating a widget in
        the usual way.
        If the 'interactive' argument is false no popups will show up to
        ask for options
        """

        # XXX: Clean up this horrible mess
        widget = self.adaptor.create(self.project.context, interactive)
        self.project.set_new_widget_name(widget)
        self.setup_widget(widget)
        self.adaptor.post_create(self.project.context, widget, interactive)
        self.adaptor.fill_empty(self.project.context, widget)

    def copy(self, project=None, keep_name=False):
        """
        Create a copy of the widget. You must specify a project since
        a name conflict resolution will be performed for the copy.

        @param project: project where the source widget
          will live in, if None the project of the gadget will be used.
        @param keep_name: True if the name should not be modified
        """

        if not project:
            project = self.project

        gadget = Gadget.from_xml(project, self.to_xml(), self.name)
        if not keep_name:
            project.set_new_widget_name(gadget.widget)
        return gadget

    def select(self):
        """
        Select this widget, i.e. clear the current selection and add
        the widget.
        """
        self.project.selection.set(self.widget)

    def to_xml(self, skip_external_references=False):
        """
        @returns: an xml representation of the gadget
        """
        from gazpacho.filewriter import XMLWriter
        xw = XMLWriter(project=self.project)
        return xw.serialize(self, skip_external_references)

    def is_toplevel(self):
        """
        @returns: True if the gadget is a toplevel
        """
        return self.adaptor.is_toplevel()

    def add_signal_handler(self, signal_handler):
        """
        Add a signal handler.

        @param signal_handler: the signal handler
        @type signal_handler: L{gazpacho.signaleditor.SignalInfo}
        """
        name = signal_handler.normalized_name
        self._signals.setdefault(name, []).append(signal_handler)
        self.emit('add-signal', signal_handler)

    def remove_signal_handler(self, signal_handler):
        """
        Remove the specified signal handler.

        @param signal_handler: the signal handler
        @type signal_handler: L{gazpacho.signaleditor.SignalInfo}
        """
        name = signal_handler.normalized_name
        if name in self._signals:
            self._signals[name].remove(signal_handler)
            self.emit('remove-signal', signal_handler)

    def change_signal_handler(self, old_signal_handler, new_signal_handler):
        old_name = old_signal_handler.normalized_name
        if old_name != new_signal_handler.normalized_name:
            return

        if not old_name in self._signals:
            return

        signals = self._signals[old_name]
        signals[signals.index(old_signal_handler)] = new_signal_handler.copy()

    def list_signal_handlers(self, signal_name):
        """
        Get a list of all signal handlers for which their normalized
        handler names matches the specified name.

        @param signal_name: the normalized signal handler name
        @type signal_name: str
        @return: a list of signal handler info objects
        @rtype: list (of L{gazpacho.signaleditor.SignalInfo})
        """
        # XXX should we normalize the name?
        return self._signals.get(signal_name, [])

    def get_all_signal_handlers(self):
        return list(self._signals.values())

    # Private

    def _create_prop(self, prop_name, child=False):
        # Get the property type
        type_name = gobject.type_name(self.widget)
        if child:
            parent = self.widget.get_parent()
        else:
            parent = None
        prop_type = prop_registry.get_prop_type(type_name, prop_name, parent)

        # Instantiate the property from the property type
        return prop_type(self)

    def _internal_get_prop(self, prop_name, child):
        if child:
            properties = self.child_properties
        else:
            properties = self.properties
        if prop_name in properties:
            return properties[prop_name]

        prop = self._create_prop(prop_name, child)
        properties[prop_name] = prop
        return prop

type_register(BaseGadget)

# XXX: Rename/Move to WidgetGadget
class Gadget(BaseGadget):
    """
    A wrapper around a gtk.Widget.
    Adds DND handlers and annotations.

    @ivar dnd_gadget: DND the widget that should be dragged
      (not necessarily this one)
    """

    def __init__(self, adaptor, project):
        BaseGadget.__init__(self, adaptor, project)

        self._dnd_drop_region = None
        self.dnd_gadget = None

    # Class methods

    #@classmethod
    def replace(cls, old_widget, new_widget, parent):
        gadget = super(Gadget, cls).replace(old_widget, new_widget, parent)
        if gadget:
            gadget._connect_signal_handlers(gadget.widget)
        return gadget
    replace = classmethod(replace)

    # Public

    def get_parent(self):
        if self.widget.flags() & gtk.TOPLEVEL:
            return None

        return get_parent(self.widget)

    def setup_widget(self, widget):
        BaseGadget.setup_widget(self, widget)

        self.widget.add_events(gdk.BUTTON_PRESS_MASK |
                               gdk.BUTTON_RELEASE_MASK |
                               gdk.KEY_PRESS_MASK |
                               gdk.POINTER_MOTION_MASK)
        if self.widget.flags() & gtk.TOPLEVEL:
            self.widget.connect('delete-event', self._on_widget__delete_event)

        self.widget.connect('popup-menu', self._on_widget__popup_menu)
        self.widget.connect('key-press-event', self._on_widget__key_press_event)

        self._connect_signal_handlers(widget)

        from gazpacho.dndhandlers import WidgetDnDHandler

        dndhandler = WidgetDnDHandler()

        # Init Drag and Drop

        # XXX we need to attach handlers for all widgets even if they
        # do not support dnd themselves (e.g. windows) since they
        # might have to handle the events for their children. This
        # should hopefully be solved in a better way at some point
        dndhandler.connect_drag_handlers(self.widget)
        dndhandler.connect_drop_handlers(self.widget)

    def setup_toplevel(self):
        """Add the action groups of Gazpacho to this toplevel and
        also set this toplevel transient for the main window.
        """
        widget = self.widget
        widget.add_accel_group(bar_manager.get_accel_group())

        # make window management easier by making created windows
        # transient for the editor window
        widget.set_transient_for(get_utility(IGazpachoApp).get_window())

    def set_drop_region(self, region):
        """Set the drop region and make sure it is painted.

        @param region: a tuple of x, y, width and height
        @type region: tuple
        """
        self._dnd_drop_region = region
        self.widget.queue_draw()

    def clear_drop_region(self):
        """Clear the drop region and make sure it is not painted anymore."""
        if self._dnd_drop_region:
            self._dnd_drop_region = None
            self.widget.queue_draw()

    # Private

    def _connect_signal_handlers(self, widget):
        # don't connect handlers for placeholders
        if isinstance(widget, Placeholder):
            return

        widget.set_redraw_on_allocate(True)

        # check if we've already connected an event handler
        if not widget.get_data('event-handler-connected'):
            # we are connecting to the event signal instead of the more
            # apropiated expose-event and button-press-event because some
            # widgets (ComboBox) does not allows us to connect to those
            # signals. See http://bugzilla.gnome.org/show_bug.cgi?id=171125
            # Hopefully we will be able to fix this hack if GTK+ is changed
            # in this case
            widget.connect('event', self._on_widget__event)
            widget.connect('event-after', self._on_widget__event_after)

#            widget.connect("expose-event", self._expose_event)
#            widget.connect("button-press-event", self._button_press_event)
            widget.set_data('event-handler-connected', 1)

        ##
        ## # we also need to get expose events for any children
        ## if isinstance(widget, gtk.Container):
        ##     widget.forall(self._connect_signal_handlers)

    def  _draw_drop_region(self, widget, expose_window, annotator=None):
        if not annotator:
            annotator = Annotator(widget, expose_window)

        color = (0, 0, 1)

        (x, y, width, height) = self._dnd_drop_region

        annotator.draw_border(x, y, width, height, color)

    def _find_deepest_child_at_position(self, toplevel, container, x, y):
        found = None
        if isinstance(container, gtk.Container):
            for widget in get_all_children(container):
                coords = toplevel.translate_coordinates(widget, x, y)
                if len(coords) != 2:
                    continue
                child_x, child_y = coords

                # sometimes the found widget is not mapped or visible
                # think about a widget in a notebook page which is not selected
                if (0 <= child_x < widget.allocation.width and
                    0 <= child_y < widget.allocation.height and
                    widget.flags() & gtk.MAPPED):
                    found = self._find_deepest_child_at_position(toplevel,
                                                                 widget, x, y)
                    if found:
                        break

        return found or Gadget.from_widget(container)

    def _retrieve_from_event(self, base, event):
        """ Returns the widget in x, y of base.
        This is needed because for widgets that does not have a
        GdkWindow the click event goes right to their parent.
        """

        x = int(event.x)
        y = int(event.y)
        window = base.get_toplevel()
        if window.flags() & gtk.TOPLEVEL:
            top_x, top_y = base.translate_coordinates(window, x, y)
            return self._find_deepest_child_at_position(window, window,
                                                        top_x, top_y)

    def _button_press_event(self, widget, event):
        """
        When a widget is clicked it is selected. If it's a right click
        a context menu will be shown. If it is a left click one of
        several actions can take place.

        If the widget is not already selected it will be selected and
        nothing more will happen. If it is selected the event will be
        passed to the actual widget.

        If the Shift modifier is used the widgets will be selected in
        a circular fashion where. the parent of the currently selected
        widget will be selected.

        If the Control modifier is used the widget will be added to or
        removed from the selection.
        """
        gadget = self._retrieve_from_event(widget, event)
        if not gadget:
            return

        # DND store the widget, we might need it later
        self.dnd_gadget = gadget

        widget = gadget.widget
        # Make sure to grab focus, since we may stop default handlers
        if widget.flags() & gtk.CAN_FOCUS:
            widget.grab_focus()

        if event.button == 1:
            if event.type is not gdk.BUTTON_PRESS:
                #only single clicks allowed, thanks
                return False

            # Shift clicking circles through the widget tree by
            # choosing the parent of the currently selected widget.
            if event.state & gdk.SHIFT_MASK:
                self.project.selection.circle(widget)
                return True

            # Control clicking adds or removes the widget from the
            # selection
            if event.state & gtk.gdk.CONTROL_MASK:
                self.project.selection.toggle(widget)
                return True

            # if it's already selected don't stop default handlers,
            # e.g toggle button
            selected = widget in self.project.selection
            self.project.selection.set(widget)
            return not selected
        elif event.button == 3:
            # first select the widget
            self.project.selection.set(widget)

            # then popup the menu
            popup = Popup(command_manager, gadget)
            popup.pop(event)
            return True

        return False

    def _button_release_event(self, widget, event):
        gadget = self._retrieve_from_event(widget, event)
        if not gadget:
            return
        return gadget.adaptor.button_release(gadget.project.context,
                                              gadget.widget, event)

    def _motion_notify_event(self, widget, event):
        gadget = self._retrieve_from_event(widget, event)
        if not gadget:
            return
        return gadget.adaptor.motion_notify(gadget.project.context,
                                             gadget.widget, event)

    def _expose_event_after(self, widget, event):
        draw_annotations(widget, event.window)

        if self._dnd_drop_region:
            self._draw_drop_region(widget, event.window)

    # Callbacks
    def _on_widget__delete_event(self, widget, event):
        return widget.hide_on_delete()

    def _on_widget__event(self, widget, event):
        """We only delegate this call to the appropiate event handler"""
        if event.type == gdk.BUTTON_PRESS:
            return self._button_press_event(widget, event)
        elif event.type == gdk.BUTTON_RELEASE:
            return self._button_release_event(widget, event)
        elif event.type == gdk.MOTION_NOTIFY:
            return self._motion_notify_event(widget, event)

        return False

    def _on_widget__event_after(self, widget, event):
        """We only delegate this call to the appropiate event handler"""
        if event.type == gdk.EXPOSE:
            self._expose_event_after(widget, event)

    def _on_widget__key_press_event(self, widget, event):
        gadget = Gadget.from_widget(widget)

        if event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete):
            # We will delete all the selected items
            if len(self.project.selection) == 1:
                gadget.project.delete_selection()
            return True

        return False

    def _on_widget__popup_menu(self, widget):
        return True # XXX TODO




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2008 Etienne Robillard <erob@gthcfoundation.org>

"""Utilities for manipulation of metadata objects"""

__all__ = ['get_mapper']

def get_mapper(entity, meta, engine, autoload=True):
    """
    Setup and return a `sqlalchemy.orm.mapper` object for
    a given entity.
    """
    from sqlalchemy.orm import class_mapper
    if not meta.is_bound():
        # bind to `engine' if not already done
        meta.bind = engine
    mapper = class_mapper(entity)
    return mapper




#!/usr/bin/env python
# Copyright (C) 2007-2009 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.

import sys
import wikiapp

from MoinMoin import log
from MoinMoin.request import request_wsgi
from MoinMoin.server import Config

from test_support import (
    BaseControllerTestCase, 
    TestClient, 
    unittest,
    make_app,
    settings
    )

if not 'wikiconfig' in sys.modules:
    default_instance = wikiapp.default_instance
    sys.path.insert(-1, default_instance)

import wikiconfig

# Disables spurious log messages while running the testsuite
# logger = log.getLogger(__name__)
# logger.root.setLevel('INFO')
class WikiConfig(Config):
    pass

class MoinMoinControllerTestCase(BaseControllerTestCase):

    wsgi_app = wikiapp.MoinMoinController
    #settingsClass = LazySettings
    
    def setUp(self):    
        #debug_level=2 enables interactive pdb hook
        self.wikiconfig = WikiConfig()
        self.callback = self.wsgi_app(settings=settings)
        self.client = TestClient(self.callback)
        
    def test_application(self):
        req = self.client.request
        response = self.client.get('/')
        self.assertEqual(response.status_code, '200 OK')
        
#def main():
#    c = WikiConfig()
#    application = WikiController(c)
#    # fastcgi here
#    server = fastcgi.ForkingWSGIServer(wsgi_app, workers=2)
#    server.serve_forever()





#!/usr/bin/env python
# -*- coding: utf-8 -*-

from webhelpers.paginate import Page

__all__ = ['paginate']

# Thanks to the webhelpers.paginate authors for providing a 
# reliable paginator algorithm
def paginate(items, settings):

    p = Page(items, page=settings.BLOGENGINE['PAGE_MAX'], 
        items_per_page=settings.BLOGENGINE['ITEMS_PER_PAGE'])

    if p.page == p.page_count:
        next_page = p.first_page
    else:
        next_page = p.page + 1
    
    if p.page == 1:
        prev_page = p.page
    else:
        prev_page = p.page - 1

    return (p, next_page, prev_page)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Another stand-alone HTTP server for serving Django apps in pure
WSGI context with the ``wsgiref`` module. 
"""

import os,sys,logging

try:
    import argparse2 as argparse
except ImportError:
    import argparse

    
from notmm.adapters.utils import make_app
from notmm.utils.django_settings import LazySettings
from notmm.http import HTTPServer, get_bind_addr, daemonize
from notmm.utils.configparse import loadconf
from notmm.utils.pastelib import eval_import

log = logging.getLogger('httpserver')
settings = LazySettings()
debug = True
    
# quick and dirty logging dispatcher
_ = lambda x: log.debug(x)

__all__ = ['auth_module', 'parse_cmdline', 'exit', 'main']

def parse_cmdline(argv):
    parser = argparse.ArgumentParser(__file__)
    parser.add_argument('-d', '--debug', action="store_true", \
        help="Enable Debugging mode", default=False)
    parser.add_argument('-c', '--config', dest="filename",
        default="development.ini", nargs=1, \
        help="Path to a development.ini-like file")
    parser.add_argument('-s', '--ssl', action="store_true", \
        dest="enable_ssl", \
        help="Enable SSL transport mode (not implemented!)")
    parser.add_argument('--disable-auth', action="store_true", \
        dest="disable_auth", \
        help="Disable oauth2 middleware backend", default=False)
    parser.add_argument('--disable-xdserver', action="store_true", \
        dest="disable_xdserver", \
        help="Disable xdserver network database backend", default=False)
    parser.add_argument('--settings', dest="disable_settings", nargs=1, \
        required=False, help="TARGET settings module", default=False)
    # add support for custom PYTHONPATH 
    parser.add_argument('--pythonpath', dest="pythonpath", default=None, \
        required=False)
    
    # show version with --version
    parser.add_argument('--version', dest="version", \
        help="Show the current version")

    # app_label hint
    parser.add_argument('app_label', type=str, \
        help="TARGET application to launch")
    
    options = parser.parse_args(args=argv[1:])
    return options

def exit(retcode):
    sys.exit(retcode)

def main(argv):
    # Start the http server after having parsed
    # options defined in the development.ini file. 

    # Read the default configuration file
    options = parse_cmdline(sys.argv)

    #if options.debug is not False: settings.DEBUG = True # override 

    if options.pythonpath is not None:
        for path in options.pythonpath.split(':'):
            #if DEBUG:
            #    print 'Path: %r' % path 
            sys.path.insert(0, path)
    
    _defaults = {
        'DEBUG': os.environ.get('DJANGO_DEBUG', debug),
        'auth_module': 'authkit',
        'WSGIHandlerClass': eval_import('notmm.controllers.wsgi:WSGIController'),
        'WSGIRequestClass': eval_import('notmm.utils.wsgilib.request:HTTPRequest')
    }

    if options.filename:
        # hack
        global_conf = loadconf(options.filename[0])
        if 'defaults' in global_conf:
            _defaults.update(global_conf['defaults'])
    else:
        raise ValueError('Please specify a valid config file path')

    app_conf = global_conf[options.app_label]
    
    # A INET4 address to bind to for listening for HTTP connections 
    # -> (host, port)
    bind_addr = get_bind_addr(global_conf)
    
    # Use command line settings module if set
    if not options.disable_settings:
        settings_module = os.environ['DJANGO_SETTINGS_MODULE']

    else:
        settings_module = options.disable_settings[0]

    if not 'controller' in app_conf:
        # Default app. To change this behavior, specify an alternative
        # "controller" param in the config file (development.ini)
        WSGIHandlerClass = _defaults['WSGIHandlerClass']
    else:
        WSGIHandlerClass = eval_import(app_conf['controller'])
    
    if settings.DEBUG: 
        _('WSGIController class=%s' % repr(WSGIHandlerClass))
        _("%d settings found in settings module: %r" % (settings.count(), settings_module))
    
    # Initialize the app with user-defined settings
    # default to 'authkit' unless --disable-authkit param
    # is given
    if not options.disable_auth and 'auth' in global_conf:
        auth_conf = global_conf['auth'] # authkit config
    else:
        auth_conf = None
    
    # This is the generic interface to the WSGIController extension
    wsgi_app = WSGIHandlerClass(
        settings=settings,
        app_conf={'django.settings_autoload': True,
                  'logging.disabled': False}
        )
    
    # Install the caching/session middleware (Beaker)
    #if settings.ENABLE_BEAKER:
    #    if 'beaker' in global_conf:
    #        session_conf = global_conf['beaker']
    #    from beaker.middleware import SessionMiddleware
    #    #Install the SessionMiddleware
    #    wsgi_app = SessionMiddleware(wsgi_app, _session_opts)
    #    _("Experimental Beaker backend initialized: %r" % wsgi_app)
    #else:
    #    _("Beaker is disabled by config. Set ENABLE_BEAKER=True to enable memcached support.")

    # Enable the authentication middleware (LoginController)
    # unless the "disable_authkit" option has been set
    if options.disable_auth is not True:
        #auth_module = _defaults['auth_module']
        from wsgi_oauth2.client import GoogleOAuthClient

        #auth_conf = global_conf[auth_module]
        #auth_conf['handle_exceptions'] = False
        client = GoogleOAuthClient(
            settings.OAUTH2_CLIENT_ID,
            access_token=settings.OAUTH2_ACCESS_TOKEN,
            scope=settings.OAUTH2_SCOPE, 
            redirect_url=settings.OAUTH2_REDIRECT_URL,
            )
        if client is not None:
            from wsgi_oauth2.controller import OAuthController    
            wsgi_app = OAuthController(
                wsgi_app, client,
                secret=settings.SECRET_KEY, 
                login_path=settings.OAUTH2_LOGIN_URL)
            _("OAuth2 middleware enabled class=%r"% type(wsgi_app))
    else:
        _("OAuth2 middleware disabled.")
    
    daemonize(wsgi_app, bind_addr)

if __name__ == "__main__":
    #assert len(sys.argv) >= 2, 'usage: runserver.py host:port' 
    main(argv=sys.argv)




# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import gobject

from gazpacho import util
from gazpacho.placeholder import Placeholder
from gazpacho.properties import prop_registry, CustomProperty, \
     StringType, TransparentProperty, BooleanType, IntType
from gazpacho.gadget import Gadget
from gazpacho.widgetregistry import widget_registry
from gazpacho.widgetadaptor import WidgetAdaptor

# this ones won't be imported at startup if we don't import it manually
from gazpacho.widgets.base import image
from gazpacho.widgets.base import spinbutton
# pyflakes
assert image and spinbutton

# GtkWidget
class EventsProp(TransparentProperty):
    default = None
prop_registry.override_simple('GtkWidget::events',  EventsProp)

prop_registry.override_simple('GtkWidget::width-request',
                              minimum=-1, maximum=10000)
prop_registry.override_simple('GtkWidget::height-request',
                              minimum=-1, maximum=10000)

class TooltipProp(TransparentProperty):
    def get(self):
        data = gtk.tooltips_data_get(self.object)
        if data is not None:
            return data[2]

    def set(self, value):
        tooltips = self._project.tooltips
        if not tooltips:
            value = None

        tooltips.set_tip(self.object, value, None)
prop_registry.override_property('GtkWidget::tooltip', TooltipProp)

class ObjectDataProp(CustomProperty):
    # name of qdata
    qdata = None
    def get(self):
        value = self.object.get_data(self.qdata)
        # If it is not set (eg None), set the value from the property
        if value is None:
            value = self.object.get_property(self.name)
            self.set(value)

        return value

    def set(self, value):
        self.object.set_data(self.qdata, value)
        self.notify()

class VisibleProp(ObjectDataProp, BooleanType):
    qdata = 'gazpacho::visible'

prop_registry.override_simple('GtkWidget::visible', VisibleProp)
prop_registry.override_simple('GtkWidget::is-focus', ObjectDataProp,
                              qdata='gazpacho::is-focus')

# We don't want has-focus to be editable, what we want to be used
# is is-focus instead.
prop_registry.override_simple('GtkWidget::has-focus', editable=False,
                              persistent=False)

prop_registry.override_simple('GtkWidget::name', translatable=False,
                              priority=0)

class SizeGroupProp(CustomProperty, StringType):
    # we can remove this class when depending in gtk 2.8 since this property
    # is readable there
    editable = True
    translatable = False
    custom = True

    def __init__(self, object):
        super(SizeGroupProp, self).__init__(object)

    def get(self):
        return self.object.get_data('gazpacho::sizegroup')

    def set(self, value):
        self.object.set_data('gazpacho::sizegroup', value)
        self.notify()

    def save(self):
        # gtkbuilder does not use this fake property
        if self._project.get_version() == 'gtkbuilder':
            return None
        return self.get() or None

prop_registry.override_property('GtkWidget::sizegroup', SizeGroupProp)

# FIXME: should be for *all* widgets, but only the onces
#        which has GtkDialog as a parent.
class ResponseIdProp(CustomProperty, IntType):
    editable = True
    translatable = False
    custom = True
    label = "Dialog response"

    def __init__(self, object):
        super(ResponseIdProp, self).__init__(object)

    def get(self):
        return self.object.get_data('gazpacho::response-id')

    def set(self, value):
        self.object.set_data('gazpacho::response-id', value)
        self.notify()

    def save(self):
        if self._project.get_version() == 'gtkbuilder':
            return None
        return super(ResponseIdProp, self).save()
prop_registry.override_property('GtkWidget::response-id', ResponseIdProp)

# GtkContainer
prop_registry.override_simple('GtkContainer::border-width',
                              minimum=0, maximum=10000)
prop_registry.override_simple('GtkContainer::resize-mode',
                              editable=False)

class ContainerAdaptor(WidgetAdaptor):
    def get_children(self, context, widget):
        children = []
        if isinstance(widget, gtk.Container):
            children += util.get_all_children(widget)
        return children

    def replace_child(self, context, current, new, container):
        if current is None:
            container.add(new)
            return

        if current.parent != container:
            return

        props = {}
        for pspec in gtk.container_class_list_child_properties(container):
            props[pspec.name] = container.child_get_property(current,
                                                             pspec.name)

        gtk.Container.remove(container, current)
        container.add(new)

        for name, value in list(props.items()):
            container.child_set_property(new, name, value)

    def fill_empty(self, context, widget):
        widget.add(Placeholder())

# GtkMisc
align_ns = dict(default=0.5, minimum=0.0, maximum=1.0,
                step_increment=0.01, page_increment=0.1,
                climb_rate=0.2)
prop_registry.override_simple('GtkMisc::xalign', **align_ns)
prop_registry.override_simple('GtkMisc::yalign', **align_ns)

# GtkPaned
class PanedAdaptor(ContainerAdaptor):
    def fill_empty(self, context, paned):
        paned.add1(Placeholder())
        paned.add2(Placeholder())
        # we want to set the position in the middle but
        # as the paned has not yet a parent it is not
        # realized we can't know its size
        gobject.idle_add(self._set_position, paned)

    def _set_position(self, paned):
        """This function get called until the paned is realized.
        Then it puts the bar in the middle.
        """
        if paned.flags() & gtk.REALIZED:
            alloc = paned.get_allocation()
            if isinstance(paned, gtk.VPaned):
                pos = alloc.height / 2
            else:
                pos = alloc.width / 2
            paned.set_position(pos)
            return False
        else:
            return True

# GtkScrolledWindow
# these widgets does not need a viewport when added to a scrolledwindow
scrollable_widgets = (gtk.TreeView, gtk.TextView, gtk.Viewport)

class ScrolledWindowAdaptor(ContainerAdaptor):

    def fill_empty(self, context, scrolledwindow):
        self._add_viewport(scrolledwindow, context.get_project())

    def _add_viewport(self, scrolledwindow, project):
        """ScrolledWindow should be empty before calling this method"""
        klass = widget_registry.get_by_name('GtkViewport')
        viewport = Gadget(klass, project)
        viewport.create_widget(interactive=False)
        scrolledwindow.add(viewport.widget)
        project.add_widget(viewport.widget)
        viewport.widget.show_all()
        return viewport

    def replace_child(self, context, current, new, container):
        global scrollable_widgets

        if current is None:
            raise ValueError("Can not replace None widget")

        # we don't care about current since it is probably different from
        # our child because we always have a viewport when adding a widget
        # that is not scrollable
        child = container.get_child()
        container.remove(child)
        project = context.get_project()
        if child is not current:
            project.remove_widget(child)

        # luckily viewports or scrolledwindow does not have any packing
        # properties

        if isinstance(new, scrollable_widgets):
            container.add(new)
        elif isinstance(new, Placeholder):
            self._add_viewport(container, project)
        else:
            raise ValueError("You can only put scrollable widgets or "
                             "placeholders inside a ScrollableWindow: "
                             "%s" % new)

# GtkViewport
class ViewportAdaptor(ContainerAdaptor):

    def replace_child(self, context, current, new, container):
        """If the new child is a scrollable widget and our parent is a
        scrolled window, we remove ourselves"""
        global scrollable_widgets
        parent = container.get_parent()
        if (isinstance(parent, gtk.ScrolledWindow)
            and isinstance(new, scrollable_widgets)):
            project = context.get_project()
            project.remove_widget(container)
            parent.remove(container)
            parent.add(new)
        else:
            ContainerAdaptor.replace_child(self, context, current, new,
                                           container)

class IconViewAdaptor(ContainerAdaptor):
    def fill_empty(self, context, widget):
        pass

# GtkLabel
class LabelAdaptor(WidgetAdaptor):
    def post_create(self, context, label, interactive):
        label.set_label(label.get_name())
        label.set_alignment(0.0, 0.5)

prop_registry.override_simple('GtkLabel::mnemonic-widget', editable=True)
prop_registry.override_simple('GtkLabel::label', priority=1)

# GtkFrame
class FrameAdaptor(ContainerAdaptor):
    def post_create(self, context, frame, interactive):
        frame.set_label(frame.get_name())
        frame.set_shadow_type(gtk.SHADOW_NONE)

prop_registry.override_simple('GtkFrame::label', default="Frame")
prop_registry.override_simple('GtkFrame::label-xalign', **align_ns)
prop_registry.override_simple('GtkFrame::label-yalign', **align_ns)

# GtkExpander
class ExpanderAdaptor(ContainerAdaptor):
    def post_create(self, context, expander, interactive):
        expander.set_label(expander.get_name())
        expander.set_expanded(True)

prop_registry.override_simple('GtkExpander::expanded', default=True)

# GtkEntry
# optional="True" optional-default="False" ?
prop_registry.override_simple('GtkEntry::width-chars', default=0)

# GtkProgressBar
# Gtk+ is broken here
p = prop_registry.get_prop_type('GtkProgressBar', 'activity-step')
# epydoc
if p is not None:
    p.minimum == 0 # HACK

# default child properties, not quite implemented
#prop_registry.override_simple('GtkLabel::expand', parent='GtkVBox',
#                              default=False)
#prop_registry.override_simple('GtkMenubar::expand', parent='GtkVBox',
#                              default=False)
#prop_registry.override_simple('GtkStatusbar::expand', parent='GtkVBox',
#                              default=False)
#prop_registry.override_simple('GtkToolbar::expand', parent='GtkVBox',
#                              default=False)
#prop_registry.override_simple('GtkVSeparator::expand', parent='GtkHBox',
#                              default=False)



